# This program demos how to use binary vectors in milvus.
import random
import sys

import numpy as np

from milvus import *

COLLECTION_NAME = "binary_collection"
DIM = 512

COLLECTION_PARAM = {
    # "collection_name": COLLECTION_NAME,
    "dimension": DIM,
    "index_file_size": 1024,
    # MetricType `HAMMING`, `JACCARD`, `TANIMOTO` are only used for binary vectors
    # "metric_type": MetricType.JACCARD
}


def gen_vectors(dim, num):
    return [[random.randint(0, 255) for _ in range(dim)] for _ in range(num)]


# gen binary vectors
def gen_binary_vectors(dim, num):
    # in binary vectors, a bit represent a dimension.
    # a value of `uint8` describe 8 dimension
    vectors = gen_vectors(dim // 8, num)
    data = np.array(vectors, dtype='uint8').astype("uint8")

    # convert to bytes
    return [bytes(vector) for vector in data]


def main(collection_name, metric_type):
    client = Milvus(host="localhost", port="19121", handler="HTTP")

    _, ok = client.has_collection(collection_name)

    if not ok:
        COLLECTION_PARAM["collection_name"] = collection_name
        COLLECTION_PARAM["metric_type"] = metric_type
        status = client.create_collection(COLLECTION_PARAM)
        if not status.OK():
            print("Create collection {} fail: {}".format(collection_name, status))
            sys.exit(1)

    datas = gen_binary_vectors(DIM, 1000)
    status, _ = client.insert(collection_name, datas)
    if 0 != status.code:
        print("Insert binary vectors into collection {} failed: {}".format(collection_name, status))
        exit(1)

    # Wait for 6 seconds, until Milvus server persist vector data.
    client.flush([collection_name])

    # select top 3 vectors for similarity search
    query_vectors = datas[0:3]

    # search param
    search_param = {
        "nprobe": 10
    }

    status, results = client.search(collection_name, top_k=2, query_records=query_vectors, params=search_param)
    if 0 != status.code:
        print("Search failed. reason:{}".format(status.message))
    else:
        print(results)

    client.drop_collection(collection_name)


if __name__ == '__main__':
    # Here is all binary metric type in list.
    binary_metrics = [MetricType.HAMMING, MetricType.JACCARD, MetricType.TANIMOTO,
                      MetricType.SUBSTRUCTURE, MetricType.SUPERSTRUCTURE]

    for metric in binary_metrics:
        collection_name = "{}_{}".format(COLLECTION_NAME, metric.name)
        print("\n\n*** Collection: {}   Metric: {} ***\n\n".format(collection_name, metric))
        main(collection_name, metric)



from enum import IntEnum


class Status:
    """
    :attribute code: int (optional) default as ok

    :attribute message: str (optional) current status message
    """

    SUCCESS = 0
    UNEXPECTED_ERROR = 1
    CONNECT_FAILED = 2
    PERMISSION_DENIED = 3
    COLLECTION_NOT_EXISTS = 4
    ILLEGAL_ARGUMENT = 5
    ILLEGAL_RANGE = 6
    ILLEGAL_DIMENSION = 7
    ILLEGAL_INDEX_TYPE = 8
    ILLEGAL_COLLECTION_NAME = 9
    ILLEGAL_TOPK = 10
    ILLEGAL_ROWRECORD = 11
    ILLEGAL_VECTOR_ID = 12
    ILLEGAL_SEARCH_RESULT = 13
    FILE_NOT_FOUND = 14
    META_FAILED = 15
    CACHE_FAILED = 16
    CANNOT_CREATE_FOLDER = 17
    CANNOT_CREATE_FILE = 18
    CANNOT_DELETE_FOLDER = 19
    CANNOT_DELETE_FILE = 20
    BUILD_INDEX_ERROR = 21
    ILLEGAL_NLIST = 22
    ILLEGAL_METRIC_TYPE = 23
    OUT_OF_MEMORY = 24

    def __init__(self, code=SUCCESS, message="Success"):
        self.code = code
        self.message = message

    def __repr__(self):
        attr_list = ['%s=%r' % (key, value)
                     for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(attr_list))

    def __eq__(self, other):
        """
        Make Status comparable with self by code
        """
        if isinstance(other, int):
            return self.code == other

        return isinstance(other, self.__class__) and self.code == other.code

    def __ne__(self, other):
        return self != other

    def OK(self):
        return self.code == Status.SUCCESS


class IndexType(IntEnum):
    """Index type enum.
    """

    #: Invalid index type.
    INVALID = 0

    #: FLAT index. See `FLAT <https://milvus.io/docs/v1.0.0/index.md#FLAT>`_.
    FLAT = 1

    #: IVF(Inverted File) FLAT index.
    #: See `IVF_FLAT <https://milvus.io/docs/v1.0.0/index.md#IVF_FLAT>`_.
    IVF_FLAT = 2

    #: IVF SQ8 index. See `IVF_SQ8 <https://milvus.io/docs/v1.0.0/index.md#IVF_SQ8>`_.
    IVF_SQ8 = 3

    #: RNSG(Refined NSG) index. See `RNSG <https://milvus.io/docs/v1.0.0/index.md#RNSG>`_.
    RNSG = 4

    #: IVF SQ8 Hybrid index. See `IVF_SQ8H <https://milvus.io/docs/v1.0.0/index.md#IVF_SQ8H>`_.
    IVF_SQ8H = 5

    #: IVF PQ index. See `IVF_PQ <https://milvus.io/docs/v1.0.0/index.md#IVF_PQ>`_.
    IVF_PQ = 6

    #: HNSW index. See `HNSW <https://milvus.io/docs/v1.0.0/index.md#HNSW>`_.
    HNSW = 11

    #: ANNOY index. See `ANNOY <https://milvus.io/docs/v1.0.0/index.md#ANNOY>`_.
    ANNOY = 12

    #: Alternative name for `IVF_FLAT`. Reserved for compatibility.
    IVFLAT = IVF_FLAT
    #: Alternative name for `IVF_SQ8H`. Reserved for compatibility.
    IVF_SQ8_H = IVF_SQ8H

    def __repr__(self):
        return "<{}: {}>".format(self.__class__.__name__, self._name_)

    def __str__(self):
        return self._name_


class MetricType(IntEnum):
    """Metric type enum.
    """

    #: Invalid metric type.
    INVALID = 0

    #: Euclidean distance. A metric for float vectors.
    #: See `Euclidean distance <https://milvus.io/docs/v1.0.0/metric.md#Euclidean-distance-L2>`_.
    L2 = 1

    #: Inner product. A metric for float vectors.
    #: See `Inner Product <https://milvus.io/docs/v1.0.0/metric.md#Inner-product-IP>`_.
    IP = 2

    #: Hamming distance. A metric for binary vectors.
    #: See `Hamming distance <https://milvus.io/docs/v1.0.0/metric.md#Hamming-distance>`_.
    HAMMING = 3

    #: Jaccard distance. A metric for binary vectors.
    #: See `Jaccard distance <https://milvus.io/docs/v1.0.0/metric.md#Jaccard-distance>`_.
    JACCARD = 4

    #: Tanimoto distance. A metric for binary vectors.
    #: See `Tanimoto distance <https://milvus.io/docs/v1.0.0/metric.md#Tanimoto-distance>`_.
    TANIMOTO = 5

    #: Superstructure. A metric for binary vectors,
    #: only support :attr:`~milvus.IndexType.FLAT` index.
    #: See `Superstructure <https://milvus.io/docs/v1.0.0/metric.md#Superstructure>`_.
    SUBSTRUCTURE = 6

    #: Substructure. A metric for binary vectors, only support :attr:`~milvus.IndexType.FLAT` index.
    #: See `Substructure <https://milvus.io/docs/v1.0.0/metric.md#Substructure>`_.
    SUPERSTRUCTURE = 7

    def __repr__(self):
        return "<{}: {}>".format(self.__class__.__name__, self._name_)

    def __str__(self):
        return self._name_



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import milvus_pb2 as milvus__pb2
from . import status_pb2 as status__pb2


class MilvusServiceStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.CreateCollection = channel.unary_unary(
        '/milvus.grpc.MilvusService/CreateCollection',
        request_serializer=milvus__pb2.CollectionSchema.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.HasCollection = channel.unary_unary(
        '/milvus.grpc.MilvusService/HasCollection',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=milvus__pb2.BoolReply.FromString,
        )
    self.DescribeCollection = channel.unary_unary(
        '/milvus.grpc.MilvusService/DescribeCollection',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=milvus__pb2.CollectionSchema.FromString,
        )
    self.CountCollection = channel.unary_unary(
        '/milvus.grpc.MilvusService/CountCollection',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=milvus__pb2.CollectionRowCount.FromString,
        )
    self.ShowCollections = channel.unary_unary(
        '/milvus.grpc.MilvusService/ShowCollections',
        request_serializer=milvus__pb2.Command.SerializeToString,
        response_deserializer=milvus__pb2.CollectionNameList.FromString,
        )
    self.ShowCollectionInfo = channel.unary_unary(
        '/milvus.grpc.MilvusService/ShowCollectionInfo',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=milvus__pb2.CollectionInfo.FromString,
        )
    self.DropCollection = channel.unary_unary(
        '/milvus.grpc.MilvusService/DropCollection',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.CreateIndex = channel.unary_unary(
        '/milvus.grpc.MilvusService/CreateIndex',
        request_serializer=milvus__pb2.IndexParam.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.DescribeIndex = channel.unary_unary(
        '/milvus.grpc.MilvusService/DescribeIndex',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=milvus__pb2.IndexParam.FromString,
        )
    self.DropIndex = channel.unary_unary(
        '/milvus.grpc.MilvusService/DropIndex',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.CreatePartition = channel.unary_unary(
        '/milvus.grpc.MilvusService/CreatePartition',
        request_serializer=milvus__pb2.PartitionParam.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.HasPartition = channel.unary_unary(
        '/milvus.grpc.MilvusService/HasPartition',
        request_serializer=milvus__pb2.PartitionParam.SerializeToString,
        response_deserializer=milvus__pb2.BoolReply.FromString,
        )
    self.ShowPartitions = channel.unary_unary(
        '/milvus.grpc.MilvusService/ShowPartitions',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=milvus__pb2.PartitionList.FromString,
        )
    self.DropPartition = channel.unary_unary(
        '/milvus.grpc.MilvusService/DropPartition',
        request_serializer=milvus__pb2.PartitionParam.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.Insert = channel.unary_unary(
        '/milvus.grpc.MilvusService/Insert',
        request_serializer=milvus__pb2.InsertParam.SerializeToString,
        response_deserializer=milvus__pb2.VectorIds.FromString,
        )
    self.GetVectorsByID = channel.unary_unary(
        '/milvus.grpc.MilvusService/GetVectorsByID',
        request_serializer=milvus__pb2.VectorsIdentity.SerializeToString,
        response_deserializer=milvus__pb2.VectorsData.FromString,
        )
    self.GetVectorIDs = channel.unary_unary(
        '/milvus.grpc.MilvusService/GetVectorIDs',
        request_serializer=milvus__pb2.GetVectorIDsParam.SerializeToString,
        response_deserializer=milvus__pb2.VectorIds.FromString,
        )
    self.Search = channel.unary_unary(
        '/milvus.grpc.MilvusService/Search',
        request_serializer=milvus__pb2.SearchParam.SerializeToString,
        response_deserializer=milvus__pb2.TopKQueryResult.FromString,
        )
    self.SearchByID = channel.unary_unary(
        '/milvus.grpc.MilvusService/SearchByID',
        request_serializer=milvus__pb2.SearchByIDParam.SerializeToString,
        response_deserializer=milvus__pb2.TopKQueryResult.FromString,
        )
    self.SearchInFiles = channel.unary_unary(
        '/milvus.grpc.MilvusService/SearchInFiles',
        request_serializer=milvus__pb2.SearchInFilesParam.SerializeToString,
        response_deserializer=milvus__pb2.TopKQueryResult.FromString,
        )
    self.Cmd = channel.unary_unary(
        '/milvus.grpc.MilvusService/Cmd',
        request_serializer=milvus__pb2.Command.SerializeToString,
        response_deserializer=milvus__pb2.StringReply.FromString,
        )
    self.DeleteByID = channel.unary_unary(
        '/milvus.grpc.MilvusService/DeleteByID',
        request_serializer=milvus__pb2.DeleteByIDParam.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.PreloadCollection = channel.unary_unary(
        '/milvus.grpc.MilvusService/PreloadCollection',
        request_serializer=milvus__pb2.PreloadCollectionParam.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.ReleaseCollection = channel.unary_unary(
        '/milvus.grpc.MilvusService/ReleaseCollection',
        request_serializer=milvus__pb2.PreloadCollectionParam.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.ReloadSegments = channel.unary_unary(
        '/milvus.grpc.MilvusService/ReloadSegments',
        request_serializer=milvus__pb2.ReLoadSegmentsParam.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.Flush = channel.unary_unary(
        '/milvus.grpc.MilvusService/Flush',
        request_serializer=milvus__pb2.FlushParam.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.Compact = channel.unary_unary(
        '/milvus.grpc.MilvusService/Compact',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.CreateHybridCollection = channel.unary_unary(
        '/milvus.grpc.MilvusService/CreateHybridCollection',
        request_serializer=milvus__pb2.Mapping.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.HasHybridCollection = channel.unary_unary(
        '/milvus.grpc.MilvusService/HasHybridCollection',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=milvus__pb2.BoolReply.FromString,
        )
    self.DropHybridCollection = channel.unary_unary(
        '/milvus.grpc.MilvusService/DropHybridCollection',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.DescribeHybridCollection = channel.unary_unary(
        '/milvus.grpc.MilvusService/DescribeHybridCollection',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=milvus__pb2.Mapping.FromString,
        )
    self.CountHybridCollection = channel.unary_unary(
        '/milvus.grpc.MilvusService/CountHybridCollection',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=milvus__pb2.CollectionRowCount.FromString,
        )
    self.ShowHybridCollections = channel.unary_unary(
        '/milvus.grpc.MilvusService/ShowHybridCollections',
        request_serializer=milvus__pb2.Command.SerializeToString,
        response_deserializer=milvus__pb2.MappingList.FromString,
        )
    self.ShowHybridCollectionInfo = channel.unary_unary(
        '/milvus.grpc.MilvusService/ShowHybridCollectionInfo',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=milvus__pb2.CollectionInfo.FromString,
        )
    self.PreloadHybridCollection = channel.unary_unary(
        '/milvus.grpc.MilvusService/PreloadHybridCollection',
        request_serializer=milvus__pb2.CollectionName.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )
    self.InsertEntity = channel.unary_unary(
        '/milvus.grpc.MilvusService/InsertEntity',
        request_serializer=milvus__pb2.HInsertParam.SerializeToString,
        response_deserializer=milvus__pb2.HEntityIDs.FromString,
        )
    self.HybridSearch = channel.unary_unary(
        '/milvus.grpc.MilvusService/HybridSearch',
        request_serializer=milvus__pb2.HSearchParam.SerializeToString,
        response_deserializer=milvus__pb2.TopKQueryResult.FromString,
        )
    self.HybridSearchInSegments = channel.unary_unary(
        '/milvus.grpc.MilvusService/HybridSearchInSegments',
        request_serializer=milvus__pb2.HSearchInSegmentsParam.SerializeToString,
        response_deserializer=milvus__pb2.TopKQueryResult.FromString,
        )
    self.GetEntityByID = channel.unary_unary(
        '/milvus.grpc.MilvusService/GetEntityByID',
        request_serializer=milvus__pb2.HEntityIdentity.SerializeToString,
        response_deserializer=milvus__pb2.HEntity.FromString,
        )
    self.GetEntityIDs = channel.unary_unary(
        '/milvus.grpc.MilvusService/GetEntityIDs',
        request_serializer=milvus__pb2.HGetEntityIDsParam.SerializeToString,
        response_deserializer=milvus__pb2.HEntityIDs.FromString,
        )
    self.DeleteEntitiesByID = channel.unary_unary(
        '/milvus.grpc.MilvusService/DeleteEntitiesByID',
        request_serializer=milvus__pb2.HDeleteByIDParam.SerializeToString,
        response_deserializer=status__pb2.Status.FromString,
        )


class MilvusServiceServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def CreateCollection(self, request, context):
    """*
    @brief This method is used to create collection

    @param CollectionSchema, use to provide collection information to be created.

    @return Status
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def HasCollection(self, request, context):
    """*
    @brief This method is used to test collection existence.

    @param CollectionName, collection name is going to be tested.

    @return BoolReply
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DescribeCollection(self, request, context):
    """*
    @brief This method is used to get collection schema.

    @param CollectionName, target collection name.

    @return CollectionSchema
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CountCollection(self, request, context):
    """*
    @brief This method is used to get collection schema.

    @param CollectionName, target collection name.

    @return CollectionRowCount
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ShowCollections(self, request, context):
    """*
    @brief This method is used to list all collections.

    @param Command, dummy parameter.

    @return CollectionNameList
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ShowCollectionInfo(self, request, context):
    """*
    @brief This method is used to get collection detail information.

    @param CollectionName, target collection name.

    @return CollectionInfo
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DropCollection(self, request, context):
    """*
    @brief This method is used to delete collection.

    @param CollectionName, collection name is going to be deleted.

    @return CollectionNameList
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CreateIndex(self, request, context):
    """*
    @brief This method is used to build index by collection in sync mode.

    @param IndexParam, index paramters.

    @return Status
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DescribeIndex(self, request, context):
    """*
    @brief This method is used to describe index

    @param CollectionName, target collection name.

    @return IndexParam
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DropIndex(self, request, context):
    """*
    @brief This method is used to drop index

    @param CollectionName, target collection name.

    @return Status
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CreatePartition(self, request, context):
    """*
    @brief This method is used to create partition

    @param PartitionParam, partition parameters.

    @return Status
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def HasPartition(self, request, context):
    """*
    @brief This method is used to test partition existence.

    @param PartitionParam, target partition.

    @return BoolReply
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ShowPartitions(self, request, context):
    """*
    @brief This method is used to show partition information

    @param CollectionName, target collection name.

    @return PartitionList
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DropPartition(self, request, context):
    """*
    @brief This method is used to drop partition

    @param PartitionParam, target partition.

    @return Status
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Insert(self, request, context):
    """*
    @brief This method is used to add vector array to collection.

    @param InsertParam, insert parameters.

    @return VectorIds
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetVectorsByID(self, request, context):
    """*
    @brief This method is used to get vectors data by id array.

    @param VectorsIdentity, target vector id array.

    @return VectorsData
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetVectorIDs(self, request, context):
    """*
    @brief This method is used to get vector ids from a segment

    @param GetVectorIDsParam, target collection and segment

    @return VectorIds
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Search(self, request, context):
    """*
    @brief This method is used to query vector in collection.

    @param SearchParam, search parameters.

    @return TopKQueryResult
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def SearchByID(self, request, context):
    """*
    @brief This method is used to query vector by id.

    @param SearchByIDParam, search parameters.

    @return TopKQueryResult
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def SearchInFiles(self, request, context):
    """*
    @brief This method is used to query vector in specified files.

    @param SearchInFilesParam, search in files paremeters.

    @return TopKQueryResult
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Cmd(self, request, context):
    """*
    @brief This method is used to give the server status.

    @param Command, command string

    @return StringReply
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeleteByID(self, request, context):
    """*
    @brief This method is used to delete vector by id

    @param DeleteByIDParam, delete parameters.

    @return status
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def PreloadCollection(self, request, context):
    """*
    @brief This method is used to preload collection/partitions

    @param PreloadCollectionParam, target collection/partitions.

    @return Status
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ReleaseCollection(self, request, context):
    """*
    @brief This method is used to release collection/partitions

    @param PreloadCollectionParam, target collection/partitions.

    @return Status
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ReloadSegments(self, request, context):
    """*
    @brief This method is used to reload collection segments

    @param ReLoadSegmentsParam, target segments information.

    @return Status
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Flush(self, request, context):
    """*
    @brief This method is used to flush buffer into storage.

    @param FlushParam, flush parameters

    @return Status
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Compact(self, request, context):
    """*
    @brief This method is used to compact collection

    @param CollectionName, target collection name.

    @return Status
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CreateHybridCollection(self, request, context):
    """*******************************New Interface*******************************************

    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def HasHybridCollection(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DropHybridCollection(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DescribeHybridCollection(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CountHybridCollection(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ShowHybridCollections(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ShowHybridCollectionInfo(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def PreloadHybridCollection(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def InsertEntity(self, request, context):
    """/////////////////////////////////////////////////////////////////

    rpc CreateIndex(IndexParam) returns (Status) {}

    rpc DescribeIndex(CollectionName) returns (IndexParam) {}

    rpc DropIndex(CollectionName) returns (Status) {}

    /////////////////////////////////////////////////////////////////

    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def HybridSearch(self, request, context):
    """TODO(yukun): will change to HQueryResult
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def HybridSearchInSegments(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetEntityByID(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetEntityIDs(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeleteEntitiesByID(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_MilvusServiceServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'CreateCollection': grpc.unary_unary_rpc_method_handler(
          servicer.CreateCollection,
          request_deserializer=milvus__pb2.CollectionSchema.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'HasCollection': grpc.unary_unary_rpc_method_handler(
          servicer.HasCollection,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=milvus__pb2.BoolReply.SerializeToString,
      ),
      'DescribeCollection': grpc.unary_unary_rpc_method_handler(
          servicer.DescribeCollection,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=milvus__pb2.CollectionSchema.SerializeToString,
      ),
      'CountCollection': grpc.unary_unary_rpc_method_handler(
          servicer.CountCollection,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=milvus__pb2.CollectionRowCount.SerializeToString,
      ),
      'ShowCollections': grpc.unary_unary_rpc_method_handler(
          servicer.ShowCollections,
          request_deserializer=milvus__pb2.Command.FromString,
          response_serializer=milvus__pb2.CollectionNameList.SerializeToString,
      ),
      'ShowCollectionInfo': grpc.unary_unary_rpc_method_handler(
          servicer.ShowCollectionInfo,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=milvus__pb2.CollectionInfo.SerializeToString,
      ),
      'DropCollection': grpc.unary_unary_rpc_method_handler(
          servicer.DropCollection,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'CreateIndex': grpc.unary_unary_rpc_method_handler(
          servicer.CreateIndex,
          request_deserializer=milvus__pb2.IndexParam.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'DescribeIndex': grpc.unary_unary_rpc_method_handler(
          servicer.DescribeIndex,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=milvus__pb2.IndexParam.SerializeToString,
      ),
      'DropIndex': grpc.unary_unary_rpc_method_handler(
          servicer.DropIndex,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'CreatePartition': grpc.unary_unary_rpc_method_handler(
          servicer.CreatePartition,
          request_deserializer=milvus__pb2.PartitionParam.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'HasPartition': grpc.unary_unary_rpc_method_handler(
          servicer.HasPartition,
          request_deserializer=milvus__pb2.PartitionParam.FromString,
          response_serializer=milvus__pb2.BoolReply.SerializeToString,
      ),
      'ShowPartitions': grpc.unary_unary_rpc_method_handler(
          servicer.ShowPartitions,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=milvus__pb2.PartitionList.SerializeToString,
      ),
      'DropPartition': grpc.unary_unary_rpc_method_handler(
          servicer.DropPartition,
          request_deserializer=milvus__pb2.PartitionParam.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'Insert': grpc.unary_unary_rpc_method_handler(
          servicer.Insert,
          request_deserializer=milvus__pb2.InsertParam.FromString,
          response_serializer=milvus__pb2.VectorIds.SerializeToString,
      ),
      'GetVectorsByID': grpc.unary_unary_rpc_method_handler(
          servicer.GetVectorsByID,
          request_deserializer=milvus__pb2.VectorsIdentity.FromString,
          response_serializer=milvus__pb2.VectorsData.SerializeToString,
      ),
      'GetVectorIDs': grpc.unary_unary_rpc_method_handler(
          servicer.GetVectorIDs,
          request_deserializer=milvus__pb2.GetVectorIDsParam.FromString,
          response_serializer=milvus__pb2.VectorIds.SerializeToString,
      ),
      'Search': grpc.unary_unary_rpc_method_handler(
          servicer.Search,
          request_deserializer=milvus__pb2.SearchParam.FromString,
          response_serializer=milvus__pb2.TopKQueryResult.SerializeToString,
      ),
      'SearchByID': grpc.unary_unary_rpc_method_handler(
          servicer.SearchByID,
          request_deserializer=milvus__pb2.SearchByIDParam.FromString,
          response_serializer=milvus__pb2.TopKQueryResult.SerializeToString,
      ),
      'SearchInFiles': grpc.unary_unary_rpc_method_handler(
          servicer.SearchInFiles,
          request_deserializer=milvus__pb2.SearchInFilesParam.FromString,
          response_serializer=milvus__pb2.TopKQueryResult.SerializeToString,
      ),
      'Cmd': grpc.unary_unary_rpc_method_handler(
          servicer.Cmd,
          request_deserializer=milvus__pb2.Command.FromString,
          response_serializer=milvus__pb2.StringReply.SerializeToString,
      ),
      'DeleteByID': grpc.unary_unary_rpc_method_handler(
          servicer.DeleteByID,
          request_deserializer=milvus__pb2.DeleteByIDParam.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'PreloadCollection': grpc.unary_unary_rpc_method_handler(
          servicer.PreloadCollection,
          request_deserializer=milvus__pb2.PreloadCollectionParam.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'ReleaseCollection': grpc.unary_unary_rpc_method_handler(
          servicer.ReleaseCollection,
          request_deserializer=milvus__pb2.PreloadCollectionParam.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'ReloadSegments': grpc.unary_unary_rpc_method_handler(
          servicer.ReloadSegments,
          request_deserializer=milvus__pb2.ReLoadSegmentsParam.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'Flush': grpc.unary_unary_rpc_method_handler(
          servicer.Flush,
          request_deserializer=milvus__pb2.FlushParam.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'Compact': grpc.unary_unary_rpc_method_handler(
          servicer.Compact,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'CreateHybridCollection': grpc.unary_unary_rpc_method_handler(
          servicer.CreateHybridCollection,
          request_deserializer=milvus__pb2.Mapping.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'HasHybridCollection': grpc.unary_unary_rpc_method_handler(
          servicer.HasHybridCollection,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=milvus__pb2.BoolReply.SerializeToString,
      ),
      'DropHybridCollection': grpc.unary_unary_rpc_method_handler(
          servicer.DropHybridCollection,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'DescribeHybridCollection': grpc.unary_unary_rpc_method_handler(
          servicer.DescribeHybridCollection,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=milvus__pb2.Mapping.SerializeToString,
      ),
      'CountHybridCollection': grpc.unary_unary_rpc_method_handler(
          servicer.CountHybridCollection,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=milvus__pb2.CollectionRowCount.SerializeToString,
      ),
      'ShowHybridCollections': grpc.unary_unary_rpc_method_handler(
          servicer.ShowHybridCollections,
          request_deserializer=milvus__pb2.Command.FromString,
          response_serializer=milvus__pb2.MappingList.SerializeToString,
      ),
      'ShowHybridCollectionInfo': grpc.unary_unary_rpc_method_handler(
          servicer.ShowHybridCollectionInfo,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=milvus__pb2.CollectionInfo.SerializeToString,
      ),
      'PreloadHybridCollection': grpc.unary_unary_rpc_method_handler(
          servicer.PreloadHybridCollection,
          request_deserializer=milvus__pb2.CollectionName.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
      'InsertEntity': grpc.unary_unary_rpc_method_handler(
          servicer.InsertEntity,
          request_deserializer=milvus__pb2.HInsertParam.FromString,
          response_serializer=milvus__pb2.HEntityIDs.SerializeToString,
      ),
      'HybridSearch': grpc.unary_unary_rpc_method_handler(
          servicer.HybridSearch,
          request_deserializer=milvus__pb2.HSearchParam.FromString,
          response_serializer=milvus__pb2.TopKQueryResult.SerializeToString,
      ),
      'HybridSearchInSegments': grpc.unary_unary_rpc_method_handler(
          servicer.HybridSearchInSegments,
          request_deserializer=milvus__pb2.HSearchInSegmentsParam.FromString,
          response_serializer=milvus__pb2.TopKQueryResult.SerializeToString,
      ),
      'GetEntityByID': grpc.unary_unary_rpc_method_handler(
          servicer.GetEntityByID,
          request_deserializer=milvus__pb2.HEntityIdentity.FromString,
          response_serializer=milvus__pb2.HEntity.SerializeToString,
      ),
      'GetEntityIDs': grpc.unary_unary_rpc_method_handler(
          servicer.GetEntityIDs,
          request_deserializer=milvus__pb2.HGetEntityIDsParam.FromString,
          response_serializer=milvus__pb2.HEntityIDs.SerializeToString,
      ),
      'DeleteEntitiesByID': grpc.unary_unary_rpc_method_handler(
          servicer.DeleteEntitiesByID,
          request_deserializer=milvus__pb2.HDeleteByIDParam.FromString,
          response_serializer=status__pb2.Status.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'milvus.grpc.MilvusService', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



import grpc


class MockGrpcError(grpc.RpcError):
    def __init__(self, code=1, details="error"):
        self._code = code
        self._details = details

    def code(self):
        return self._code

    def details(self):
        return self._details



import pytest
import random

import numpy as np

from milvus import ParamError

from factorys import bin_records_factory

dim = 128


class TestInsert:
    vectors = [[random.random() for _ in range(dim)] for _ in range(10000)]

    def test_insert_normal(self, gcon, gcollection):
        status, ids = gcon.insert(gcollection, self.vectors)

        assert status.OK()
        assert len(ids) == len(self.vectors)

    def test_insert_with_ids(self, gcon, gcollection):
        ids = [i for i in range(10000)]
        status, ids_ = gcon.insert(gcollection, self.vectors, ids)

        assert status.OK()
        assert len(ids_) == len(self.vectors)
        assert ids == ids_

    def test_insert_with_numpy(self, gcon, gcollection):
        vectors = np.random.rand(10000, dim).astype(np.float32)
        status, _ = gcon.insert(gcollection, vectors)
        assert status.OK(), status.message

    def test_insert_with_partition(self, gcon, gcollection):
        status = gcon.create_partition(gcollection, "tag01")
        assert status.OK()

        status, ids = gcon.insert(gcollection, self.vectors, partition_tag="tag01")

        assert status.OK()
        assert len(ids) == len(self.vectors)

    def test_insert_async(self, gcon, gcollection, ghandler):
        if ghandler == "HTTP":
            pytest.skip("HTTP not support async invoke")

        future = gcon.insert(gcollection, self.vectors, _async=True)

        status, ids = future.result()
        assert status.OK()
        assert len(ids) == len(self.vectors)

    def test_insert_async_callback(self, gcon, gcollection, ghandler):
        if ghandler == "HTTP":
            pytest.skip("HTTP not support async invoke")

        def cb(status, ids):
            assert status.OK()
            assert len(ids) == len(self.vectors)

        future = gcon.insert(gcollection, self.vectors, _async=True, _callback=cb)
        status, _ = future.result()
        assert status.OK()
        future.done()

    # @pytest.mark.parametrize("vectors", [[], None, "", 12344, [1111], [[]], [[], [1.0, 2.0]]])
    @pytest.mark.parametrize("vectors", [[1111]])
    def test_insert_invalid_vectors(self, vectors, gcon, gcollection):
        with pytest.raises(ParamError):
            gcon.insert(gcollection, vectors)

    @pytest.mark.parametrize("ids", [(), "abc", [], [1, 2], [[]]])
    def test_insert_invalid_ids(self, ids, gcon, gcollection):
        with pytest.raises(ParamError):
            gcon.insert(gcollection, self.vectors, ids)

    @pytest.mark.parametrize("tag", [[], 123])
    def test_insert_invalid_tag(self, tag, gcon, gcollection):
        with pytest.raises(ParamError):
            gcon.insert(gcollection, self.vectors, partition_tag=tag)


class TestInsertBinary:
    def test_insert_binary_normal(self, gcon, gbcollection):
        bin_vectors = bin_records_factory(dim, 10000)
        status, _ = gcon.insert(gbcollection, bin_vectors)
        assert status.OK(), status.message



import random
import sys
import time

from milvus import *

if __name__ == '__main__':
    dimension = 128
    collection_name = "insert_fail"

    insert_vectors = [[random.random() for _ in range(dimension)] for _ in range(100)]

    client = Milvus()

    client.create_collection({"collection_name": collection_name, "dimension": dimension})
    start_time = time.time()
    i = 0
    while time.time() < start_time + 2 * 24 * 3600:
        i = i + 1
        # logger.debug(i)
        # logger.debug("Row count: %d" % milvus_instance.count())
        _, count = client.count_entities(collection_name)
        print("Row count: {}".format(count))
        status, res = client.insert(collection_name, insert_vectors)
        if not status.OK():
            print("Insert failed. {}".format(status))
            sys.exit(1)
        time.sleep(0.1)



import pytest

from milvus import ParamError


class TestCreatePartition:
    def test_create_partition_normal(self, gcon, gcollection):
        status = gcon.create_partition(gcollection, "new_tag")
        assert status.OK()

    @pytest.mark.parametrize("tag", [[], 1234353, {}, [1, 2]])
    def test_create_partition_invalid_tag(self, tag, gcon, gcollection):
        with pytest.raises(ParamError):
            gcon.create_partition(gcollection, tag)

    @pytest.mark.skip(reason="Bug here. See #1762(https://github.com/milvus-io/milvus/issues/1762)")
    def test_create_partition_default(self, gcon, gcollection):
        status = gcon.create_partition(gcollection, "_default")
        assert not status.OK()

    def test_create_partition_repeat(self, gcon, gcollection):
        status = gcon.create_partition(gcollection, "tag01")
        assert status.OK()

        status = gcon.create_partition(gcollection, "tag01")
        assert not status.OK()


class TestHadPartition:
    @pytest.mark.skip(reason="default partition tag is not exists")
    def test_has_partition_default(self, gcon, gcollection):
        status, exits = gcon.has_partition(gcollection, "_default")
        assert status.OK()
        assert exits

    def test_has_partition_created(self, gcon, gcollection):
        status = gcon.create_partition(gcollection, "test")
        assert status.OK()
        status, exists = gcon.has_partition(gcollection, "test")
        assert status.OK()
        assert exists

    def test_has_partition_with_tag_non_existent(self, gcon, gcollection):
        status, exists = gcon.has_partition(gcollection, "test_has_partition_with_tag_non_existent")
        assert status.OK()
        assert not exists

    @pytest.mark.parametrize("tag", [[], None, 123, {}, True])
    def test_drop_partition_invalid_tag(self, tag, gcon, gcollection):
        with pytest.raises(ParamError):
            gcon.drop_partition(gcollection, tag)


class TestShowPartitions:
    def test_list_partitions_normal(self, gcon, gcollection):
        status = gcon.create_partition(gcollection, "tag01")
        assert status.OK()

        status, partitions = gcon.list_partitions(gcollection)
        assert status.OK()
        for partition in partitions:
            assert partition.collection_name == gcollection
            assert partition.tag in ("tag01", "_default")


class TestDropPartition:
    def test_drop_partition(self, gcon, gcollection):
        status = gcon.create_partition(gcollection, "tag01")
        assert status.OK()

        status = gcon.drop_partition(gcollection, "tag01")
        assert status.OK()

    @pytest.mark.parametrize("tag", [[], None, 123, {}])
    def test_drop_partition_invalid_tag(self, tag, gcon, gcollection):
        with pytest.raises(ParamError):
            gcon.drop_partition(gcollection, tag)

    def test_drop_partition_non_existent(self, gcon, gcollection):
        status = gcon.drop_partition(gcollection, "non_existent")
        assert not status.OK()



# -*- coding: UTF-8 -*-

import collections
import copy
import functools
import logging

from urllib.parse import urlparse

from . import __version__
from .types import IndexType, MetricType, Status
from .check import check_pass_param, is_legal_host, is_legal_port
from .pool import ConnectionPool, SingleConnectionPool, SingletonThreadPool
from .exceptions import ParamError, DeprecatedError

from ..settings import DefaultConfig as config

LOGGER = logging.getLogger(__name__)


def deprecated(func):
    @functools.wraps(func)
    def inner(*args, **kwargs):
        error_str = "Function {} has been deprecated".format(func.__name__)
        LOGGER.error(error_str)
        raise DeprecatedError(error_str)

    return inner


def check_connect(func):
    @functools.wraps(func)
    def inner(self, *args, **kwargs):
        return func(self, *args, **kwargs)

    return inner


def _pool_args(**kwargs):
    pool_kwargs = dict()
    for k, v in kwargs.items():
        if k in ("pool_size", "wait_timeout", "handler", "try_connect", "pre_ping", "max_retry"):
            pool_kwargs[k] = v

    return pool_kwargs


def _set_uri(host, port, uri, handler="GRPC"):
    default_port = config.GRPC_PORT if handler == "GRPC" else config.HTTP_PORT
    default_uri = config.GRPC_URI if handler == "GRPC" else config.HTTP_URI

    if host is not None:
        _port = port if port is not None else default_port
        _host = host
        if handler == "HTTP":
            _proto = "https" if _port == 443 else "http"
        else:
            _proto = "tcp"
    elif port is None:
        try:
            _uri = urlparse(uri) if uri else urlparse(default_uri)
            _host = _uri.hostname
            _port = _uri.port
            _proto = _uri.scheme
        except (AttributeError, ValueError, TypeError) as e:
            raise ParamError("uri is illegal: {}".format(e))
    else:
        raise ParamError("Param is not complete. Please invoke as follow:\n"
                         "\t(host = ${HOST}, port = ${PORT})\n"
                         "\t(uri = ${URI})\n")

    if not is_legal_host(_host) or not is_legal_port(_port):
        raise ParamError("host {} or port {} is illegal".format(_host, _port))

    return "{}://{}:{}".format(str(_proto), str(_host), str(_port))


class Milvus:
    def __init__(self, host=None, port=None, handler="GRPC", pool="SingletonThread", **kwargs):
        """Constructor method
        """
        self._name = kwargs.get('name', None)
        self._uri = None
        self._status = None
        self._connected = False
        self._handler = handler

        #
        self._conn = None

        _uri = kwargs.get('uri', None)
        pool_uri = _set_uri(host, port, _uri, self._handler)
        pool_kwargs = _pool_args(handler=handler, **kwargs)
        # self._pool = SingleConnectionPool(pool_uri, **pool_kwargs)
        if pool == "QueuePool":
            self._pool = ConnectionPool(pool_uri, **pool_kwargs)
        elif pool == "SingletonThread":
            self._pool = SingletonThreadPool(pool_uri, **pool_kwargs)
        elif pool == "Singleton":
            self._pool = SingleConnectionPool(pool_uri, **pool_kwargs)
        else:
            raise ParamError("Unknown pool value: {}".format(pool))

        # store extra key-words arguments
        self._kw = kwargs
        self._hooks = collections.defaultdict()

    def __enter__(self):
        self._conn = self._pool.fetch()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._conn.close()
        self._conn = None

    def __del__(self):
        return self.close()

    def _connection(self):
        return self._pool.fetch()

    @deprecated
    def set_hook(self, **kwargs):
        """
        Deprecated
        """
        # TODO: may remove it.
        if self._stub:
            self._stub.set_hook(**kwargs)
        else:
            self._hooks.update(kwargs)

    @property
    def name(self):
        return self._name

    @property
    def handler(self):
        return self._handler

    @deprecated
    def connect(self, host=None, port=None, uri=None, timeout=2):
        """
        Deprecated
        """
        if self.connected() and self._connected:
            return Status(message="You have already connected {} !".format(self._uri),
                          code=Status.CONNECT_FAILED)

        if self._stub is None:
            self._init(host, port, uri, handler=self._handler)

        if self.ping(timeout):
            self._status = Status(message="Connected")
            self._connected = True
            return self._status

        return Status()

    @deprecated
    def connected(self):
        """
        Deprecated
        """
        return self._status and self._status.OK()

    @deprecated
    def disconnect(self):
        """
        Deprecated
        """
        pass

    def close(self):
        """
        Close client instance
        """
        self._pool = None

    def client_version(self):
        """
        Returns the version of the client.

        :return: Version of the client.

        :rtype: (str)
        """
        return __version__

    def server_status(self, timeout=30):
        """
        Returns the status of the Milvus server.

        :return:
            Status: Whether the operation is successful.

            str : Status of the Milvus server.

        :rtype: (Status, str)
        """
        return self._cmd("status", timeout)

    def server_version(self, timeout=30):
        """
        Returns the version of the Milvus server.

        :return:
           Status: Whether the operation is successful.

           str : Version of the Milvus server.

        :rtype: (Status, str)
        """

        return self._cmd("version", timeout)

    @check_connect
    def _cmd(self, cmd, timeout=30):
        check_pass_param(cmd=cmd)

        with self._connection() as handler:
            return handler._cmd(cmd, timeout)

    @check_connect
    def create_collection(self, param, timeout=30):
        """
        Creates a collection.

        :type  param: dict
        :param param: Information needed to create a collection. It contains items:

            * *collection_name* (``str``) -- Collection name.
            * *dimension* (``int``) -- Dimension of embeddings stored in collection.
            * *index_file_size* (``int``) -- Segment size. See
              `Storage Concepts <https://milvus.io/docs/v1.0.0/storage_concept.md>`_.
            * *metric_type* (``MetricType``) -- Distance Metrics type. Valued form
              :class:`~milvus.MetricType`. See
              `Distance Metrics <https://milvus.io/docs/v1.0.0/metric.md>`_.


            A demo is as follow:

            .. code-block:: python

                param={'collection_name': 'name',
                   'dimension': 16,
                   'index_file_size': 1024 # Optional, default 1024，
                   'metric_type': MetricType.L2 # Optional, default MetricType.L2
                  }


        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status. Succeed if `Status.OK()` is `True`.
        :rtype: Status
        """
        if not isinstance(param, dict):
            raise ParamError('Param type incorrect, expect {} but get {} instead'
                             .format(type(dict), type(param)))

        collection_param = copy.deepcopy(param)

        if 'collection_name' not in collection_param:
            raise ParamError('collection_name is required')
        collection_name = collection_param["collection_name"]
        collection_param.pop('collection_name')

        if 'dimension' not in collection_param:
            raise ParamError('dimension is required')
        dim = collection_param["dimension"]
        collection_param.pop("dimension")

        index_file_size = collection_param.get('index_file_size', 1024)
        collection_param.pop('index_file_size', None)

        metric_type = collection_param.get('metric_type', MetricType.L2)
        collection_param.pop('metric_type', None)

        check_pass_param(collection_name=collection_name, dimension=dim,
                         index_file_size=index_file_size, metric_type=metric_type)

        with self._connection() as handler:
            return handler.create_collection(collection_name, dim, index_file_size,
                                             metric_type, collection_param, timeout)

    @check_connect
    def has_collection(self, collection_name, timeout=30):
        """

        Checks whether a collection exists.

        :param collection_name: Name of the collection to check.
        :type  collection_name: str
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status and the flag indicating if collection exists. Succeed
                 if `Status.OK()` is `True`. If status is not OK, the flag is always `False`.
        :rtype: Status, bool

        """
        check_pass_param(collection_name=collection_name)
        with self._connection() as handler:
            return handler.has_collection(collection_name, timeout)

    @check_connect
    def get_collection_info(self, collection_name, timeout=30):
        """
        Returns information of a collection.

        :type  collection_name: str
        :param collection_name: Name of the collection to describe.
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status and collection information. Succeed if `Status.OK()`
                 is `True`. If status is not OK, the returned information is always `None`.
        :rtype: Status, CollectionSchema
        """
        check_pass_param(collection_name=collection_name)
        with self._connection() as handler:
            return handler.describe_collection(collection_name, timeout)

    @check_connect
    def count_entities(self, collection_name, timeout=30):
        """
        Returns the number of vectors in a collection.

        :type  collection_name: str
        :param collection_name: target table name.
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status and row count. Succeed if `Status.OK()` is `True`.
                 If status is not OK, the returned value of is always `None`.
        :rtype: Status, int
        """
        check_pass_param(collection_name=collection_name)
        with self._connection() as handler:
            return handler.count_collection(collection_name, timeout)

    @check_connect
    def list_collections(self, timeout=30):
        """
        Returns collection list.

        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status and collection name list. Succeed if `Status.OK()` is `True`.
                 If status is not OK, the returned name list is always `[]`.
        :rtype: Status, list[str]
        """
        with self._connection() as handler:
            return handler.show_collections(timeout)

    @check_connect
    def get_collection_stats(self, collection_name, timeout=30):
        """

        Returns collection statistics information.

        :type  collection_name: str
        :param collection_name: target table name.
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status and collection statistics information. Succeed
                 if `Status.OK()` is `True`. If status is not OK, the returned information
                 is always `[]`.
        :rtype: Status, dict
        """
        check_pass_param(collection_name=collection_name)
        with self._connection() as handler:
            return handler.show_collection_info(collection_name, timeout)

    @check_connect
    def load_collection(self, collection_name, partition_tags=None, timeout=None):
        """
        Loads a collection for caching.

        :param collection_name: collection to load
        :type collection_name: str
        :param partition_tags: partition tag list. `None` indicates to load whole collection,
                               otherwise to load specified partitions.
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status. Succeed if `Status.OK()` is `True`.
        :rtype: Status
        """
        check_pass_param(collection_name=collection_name)
        with self._connection() as handler:
            return handler.preload_collection(collection_name, partition_tags, timeout)

    @check_connect
    def release_collection(self, collection_name, partition_tags=None, timeout=None):
        """
        Release a collection from memory and cache.

        :param collection_name: collection to release
        :type collection_name: str
        :param partition_tags: partition tag list. `None` indicates to release whole collection,
                               otherwise to release specified partitions.
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status. Succeed if `Status.OK()` is `True`.
        :rtype: Status
        """
        check_pass_param(collection_name=collection_name)
        with self._connection() as handler:
            return handler.release_collection(collection_name, partition_tags, timeout)

    @check_connect
    def reload_segments(self, collection_name, segment_ids, timeout=None):
        """
        Reloads segment DeletedDocs data to cache. This API is not recommended for users.

        :param collection_name: Name of the collection being deleted
        :type  collection_name: str
        :param segment_ids: Segment IDs.
        :type segment_ids: list[str]
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status. Succeed if `Status.OK()` is `True`.
        :rtype: Status
        """
        check_pass_param(collection_name=collection_name)
        with self._connection() as handler:
            return handler.reload_segments(collection_name, segment_ids, timeout)

    @check_connect
    def drop_collection(self, collection_name, timeout=30):
        """
        Deletes a collection by name.

        :param collection_name: Name of the collection being deleted
        :type  collection_name: str
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status. Succeed if `Status.OK()` is `True`.
        :rtype: Status
        """
        check_pass_param(collection_name=collection_name)
        with self._connection() as handler:
            return handler.drop_collection(collection_name, timeout)

    @check_connect
    def insert(self, collection_name, records, ids=None, partition_tag=None,
               params=None, timeout=None, **kwargs):
        """
        Insert vectors to a collection.

        :type  collection_name: str
        :param collection_name: Name of the collection to insert vectors to.
        :param ids: ID list. `None` indicates ID is generated by server system. Note that if the
                    first time when insert() is invoked ids is not passed into this method, each
                    of the rest time when inset() is invoked ids is not permitted to pass,
                    otherwise server will return an error and the insertion process will fail.
                    And vice versa.
        :type  ids: list[int]
        :param records: List of vectors to insert.
        :type  records: list[list[float]]
        :param partition_tag: Tag of a partition.
        :type partition_tag: str or None. If partition_tag is None, vectors will be inserted to the
                             default partition `_default`.
        :param params: Insert param. Reserved.
        :type params: dict
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float
        :param kwargs:
            * *_async* (``bool``) --
              Indicate if invoke asynchronously. When value is true, method returns a InsertFuture
              object; otherwise, method returns results from server.
            * *_callback* (``function``) --
              The callback function which is invoked after server response successfully. It only
              takes effect when _async is set to True.

        :return: The operation status and IDs of inserted entities. Succeed if `Status.OK()`
                 is `True`. If status is not OK, the returned IDs is always `[]`.
        :rtype: Status, list[int]
        """
        if kwargs.get("insert_param", None) is not None:
            with self._connection() as handler:
                return handler.insert(None, None, timeout=timeout, **kwargs)

        check_pass_param(collection_name=collection_name, records=records)
        _ = partition_tag is not None and check_pass_param(partition_tag=partition_tag)
        if ids is not None:
            check_pass_param(ids=ids)
            if len(records) != len(ids):
                raise ParamError("length of vectors do not match that of ids")

        params = params or dict()
        if not isinstance(params, dict):
            raise ParamError("Params must be a dictionary type")
        with self._connection() as handler:
            return handler.insert(collection_name, records, ids, partition_tag,
                                  params, timeout, **kwargs)

    def get_entity_by_id(self, collection_name, ids, timeout=None, partition_tag=None):
        """
        Returns raw vectors according to ids.

        :param collection_name: Name of the collection
        :type collection_name: str

        :param ids: list of vector id
        :type ids: list
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :param partition_tag: The partition tag of entity
        :type partition_tag: str

        :return: The operation status and entities. Succeed if `Status.OK()` is `True`.
                 If status is not OK, the returned entities is always `[]`.
        :rtype: Status, list[list[float]]
        """
        check_pass_param(collection_name=collection_name, ids=ids)
        _ = partition_tag is None or check_pass_param(partition_tag=partition_tag)

        with self._connection() as handler:
            return handler.get_vectors_by_ids(collection_name, ids, timeout=timeout,
                                              partition_tag=partition_tag)

    @check_connect
    def list_id_in_segment(self, collection_name, segment_name, timeout=None):
        """
        Get IDs of entity stored in the specified segment.

        :param collection_name: Collection the segment belongs to.
        :type collection_name: str
        :param segment_name: Segment name.
        :type segment_name: str
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status and entity IDs. Succeed if `Status.OK()` is `True`.
                 If status is not OK, the returned IDs is always `[]`.
        :rtype: Status, list[int]
        """
        check_pass_param(collection_name=collection_name)
        check_pass_param(collection_name=segment_name)
        with self._connection() as handler:
            return handler.get_vector_ids(collection_name, segment_name, timeout)

    @check_connect
    def create_index(self, collection_name, index_type=None, params=None, timeout=None, **kwargs):
        """
        Creates index for a collection.

        :param collection_name: Collection used to create index.
        :type collection_name: str
        :param index_type: index params. See `index params <param.html>`_ for supported indexes.
        :type index_type: IndexType
        :param params: Index param. See `index params <param.html>`_ for detailed index param of
                       supported indexes.
        :type params: dict
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float
        :param kwargs:
            * *_async* (``bool``) --
              Indicate if invoke asynchronously. When value is true, method returns a IndexFuture
              object; otherwise, method returns results from server.
            * *_callback* (``function``) --
              The callback function which is invoked after server response successfully. It only
              takes effect when _async is set to True.

        :return: The operation status. Succeed if `Status.OK()` is `True`.
        :rtype: Status
        """
        _index_type = IndexType.FLAT if index_type is None else index_type
        check_pass_param(collection_name=collection_name, index_type=_index_type)

        params = params or dict()
        if not isinstance(params, dict):
            raise ParamError("Params must be a dictionary type")
        with self._connection() as handler:
            return handler.create_index(collection_name, _index_type, params, timeout, **kwargs)

    @check_connect
    def get_index_info(self, collection_name, timeout=30):
        """
        Show index information of a collection.

        :type collection_name: str
        :param collection_name: table name been queried
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status and index info. Succeed if `Status.OK()` is `True`.
                 If status is not OK, the returned index info is always `None`.
        :rtype: Status, IndexParam

        """
        check_pass_param(collection_name=collection_name)

        with self._connection() as handler:
            return handler.describe_index(collection_name, timeout)

    @check_connect
    def drop_index(self, collection_name, timeout=30):
        """
        Removes an index.

        :param collection_name: target collection name.
        :type collection_name: str
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status. Succeed if `Status.OK()` is `True`.
        :rtype: Status
        """
        check_pass_param(collection_name=collection_name)

        with self._connection() as handler:
            return handler.drop_index(collection_name, timeout)

    @check_connect
    def create_partition(self, collection_name, partition_tag, timeout=30):
        """
        create a partition for a collection.

        :param collection_name: Name of the collection.
        :type  collection_name: str
        :param partition_tag: Name of the partition tag.
        :type  partition_tag: str
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status. Succeed if `Status.OK()` is `True`.
        :rtype: Status

        """
        check_pass_param(collection_name=collection_name, partition_tag=partition_tag)
        with self._connection() as handler:
            return handler.create_partition(collection_name, partition_tag, timeout)

    @check_connect
    def has_partition(self, collection_name, partition_tag, timeout=30):
        """
        Check if specified partition exists.

        :param collection_name: target table name.
        :type  collection_name: str
        :param partition_tag: partition tag.
        :type  partition_tag: str
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :returns: The operation status and a flag indicating if partition exists. Succeed
                  if `Status.OK()` is `True`. If status is not ok, the flag is always `False`.
        :rtype: Status, bool

        """
        check_pass_param(collection_name=collection_name, partition_tag=partition_tag)
        with self._connection() as handler:
            return handler.has_partition(collection_name, partition_tag, timeout)

    @check_connect
    def list_partitions(self, collection_name, timeout=30):
        """
        Show all partitions in a collection.

        :param collection_name: target table name.
        :type  collection_name: str
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :returns: The operation status and partition list. Succeed if `Status.OK()` is `True`.
                  If status is not OK, returned partition list is `[]`.
        :rtype: Status, list[PartitionParam]

        """
        check_pass_param(collection_name=collection_name)

        with self._connection() as handler:
            return handler.show_partitions(collection_name, timeout)

    @check_connect
    def drop_partition(self, collection_name, partition_tag, timeout=30):
        """
        Deletes a partition in a collection.

        :param collection_name: Collection name.
        :type  collection_name: str
        :param partition_tag: Partition name.
        :type  partition_tag: str
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float

        :return: The operation status. Succeed if `Status.OK()` is `True`.
        :rtype: Status

        """
        check_pass_param(collection_name=collection_name, partition_tag=partition_tag)
        with self._connection() as handler:
            return handler.drop_partition(collection_name, partition_tag, timeout)

    @check_connect
    def search(self, collection_name, top_k, query_records, partition_tags=None,
               params=None, timeout=None, **kwargs):
        """
        Search vectors in a collection.

        :param collection_name: Name of the collection.
        :type  collection_name: str
        :param top_k: number of vectors which is most similar with query vectors
        :type  top_k: int
        :param query_records: vectors to query
        :type  query_records: list[list[float32]]
        :param partition_tags: tags to search. `None` indicates to search in whole collection.
        :type  partition_tags: list
        :param params: Search params. The params is related to index type the collection is built.
                       See `index params <param.html>`_ for more detailed information.
        :type  params: dict
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float
        :param kwargs:
            * *_async* (``bool``) --
              Indicate if invoke asynchronously. When value is true, method returns a SearchFuture
              object; otherwise, method returns results from server.
            * *_callback* (``function``) --
              The callback function which is invoked after server response successfully. It only
              takes effect when _async is set to True.

        :returns: The operation status and search result. See <a>here</a> to find how to handle
                  search result. Succeed if `Status.OK()` is `True`. If status is not OK,
                  results is always `None`.
        :rtype: Status, TopKQueryResult
        """
        check_pass_param(collection_name=collection_name, topk=top_k, records=query_records)
        if partition_tags is not None:
            check_pass_param(partition_tag_array=partition_tags)

        params = dict() if params is None else params
        if not isinstance(params, dict):
            raise ParamError("Params must be a dictionary type")
        with self._connection() as handler:
            return handler.search(collection_name, top_k, query_records,
                                  partition_tags, params, timeout, **kwargs)

    @check_connect
    def search_in_segment(self, collection_name, file_ids, query_records, top_k,
                          params=None, timeout=None, **kwargs):
        """
        Searches for vectors in specific segments of a collection.
        This API is not recommended for users.

        The Milvus server stores vector data into multiple files. Searching for vectors in specific
        files is a method used in Mishards. Obtain more detail about Mishards, see
        `Mishards <https://github.com/milvus-io/milvus/tree/master/shards>`_.

        :param collection_name: table name been queried
        :type  collection_name: str
        :param file_ids: Specified files id array
        :type  file_ids: list[str] or list[int]
        :param query_records: all vectors going to be queried
        :type  query_records: list[list[float]]
        :param top_k: how many similar vectors will be searched
        :type  top_k: int
        :param params: Search params. The params is related to index type the collection is built.
                       See <a></a> for more detailed information.
        :type  params: dict
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float
        :param kwargs:
            * *_async* (``bool``) --
              Indicate if invoke asynchronously. When value is true, method returns a SearchFuture
              object; otherwise, method returns results from server.
            * *_callback* (``function``) --
              The callback function which is invoked after server response successfully. It only
              takes effect when _async is set to True.

        :returns: The operation status and search result. See <a>here</a> to find how to handle
                  search result. Succeed if `Status.OK()` is `True`. If status is not OK, results
                  is always `None`.
        :rtype: Status, TopKQueryResult
        """
        check_pass_param(collection_name=collection_name, topk=top_k,
                         records=query_records, ids=file_ids)

        params = dict() if params is None else params
        if not isinstance(params, dict):
            raise ParamError("Params must be a dictionary type")
        with self._connection() as handler:
            return handler.search_in_files(collection_name, file_ids,
                                           query_records, top_k, params, timeout, **kwargs)

    @check_connect
    def delete_entity_by_id(self, collection_name, id_array, timeout=None, partition_tag=None):
        """
        Deletes vectors in a collection by vector ID.

        :param collection_name: Name of the collection.
        :type  collection_name: str
        :param id_array: list of vector id
        :type  id_array: list[int]
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float
        :param partition_tag: The partition tag of entity
        :type partition_tag: str

        :returns: The operation status. If the specified ID doesn't exist, Milvus server skip it
                  and try to delete next entities, which is regard as one successful operation.
                  Succeed if `Status.OK()` is `True`.
        :rtype: Status
        """
        check_pass_param(collection_name=collection_name, ids=id_array)
        _ = partition_tag is None or check_pass_param(partition_tag=partition_tag)
        with self._connection() as handler:
            return handler.delete_by_id(collection_name, id_array, timeout, partition_tag)

    @check_connect
    def flush(self, collection_name_array=None, timeout=None, **kwargs):
        """
        Flushes vector data in one collection or multiple collections to disk.

        :type  collection_name_array: list
        :param collection_name_array: Name of one or multiple collections to flush.
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float
        :param kwargs:
            * *_async* (``bool``) --
              Indicate if invoke asynchronously. When value is true, method returns a FlushFuture
              object; otherwise, method returns results from server.
            * *_callback* (``function``) --
              The callback function which is invoked after server response successfully. It only
              takes effect when _async is set to True.

        :returns: The operation status. Succeed if `Status.OK()` is `True`.
        :rtype: Status
        """

        if collection_name_array in (None, []):
            with self._connection() as handler:
                return handler.flush([], timeout)

        if not isinstance(collection_name_array, list):
            raise ParamError("Collection name array must be type of list")

        if len(collection_name_array) <= 0:
            raise ParamError("Collection name array is not allowed to be empty")

        for name in collection_name_array:
            check_pass_param(collection_name=name)
        with self._connection() as handler:
            return handler.flush(collection_name_array, timeout, **kwargs)

    @check_connect
    def compact(self, collection_name, timeout=None, **kwargs):
        """
        Compacts segments in a collection. This function is recommended after deleting vectors.

        :type  collection_name: str
        :param collection_name: Name of the collections to compact.
        :param timeout: An optional duration of time in seconds to allow for the RPC. When timeout
                        is set to None, client waits until server responses or error occurs.
        :type  timeout: float
        :param kwargs:
            * *_async* (``bool``) --
              Indicate if invoke asynchronously. When value is true, method returns a CompactFuture
              object; otherwise, method returns results from server.
            * *_callback* (``function``) --
              The callback function which is invoked after server response successfully. It only
              takes effect when _async is set to True.

        :returns: The operation status. Succeed if `Status.OK()` is `True`.
        :rtype: Status
        """
        check_pass_param(collection_name=collection_name)
        with self._connection() as handler:
            return handler.compact(collection_name, timeout, **kwargs)

    def get_config(self, parent_key, child_key):
        """
        Gets Milvus configurations.

        """
        cmd = "get_config {}.{}".format(parent_key, child_key)

        return self._cmd(cmd)

    def set_config(self, parent_key, child_key, value):
        """
        Sets Milvus configurations.

        """
        cmd = "set_config {}.{} {}".format(parent_key, child_key, value)

        return self._cmd(cmd)



# This program demos how to connect to Milvus vector database,
# create a vector collection,
# insert 10 vectors,
# and execute a vector similarity search.
import datetime
import sys
import random
from milvus import Milvus, IndexType, MetricType

# Milvus server IP address and port.
# You may need to change _HOST and _PORT accordingly.
_HOST = '127.0.0.1'
_PORT = '19530'  # default value

# Vector parameters
_DIM = 128  # dimension of vector


def main():
    # Specify server addr when create milvus client instance
    milvus = Milvus(_HOST, _PORT)

    # Create collection demo_collection if it dosen't exist.
    collection_name = 'example_async_collection_'

    status, ok = milvus.has_collection(collection_name)
    if not ok:
        param = {
            'collection_name': collection_name,
            'dimension': _DIM,
            'index_file_size': 128,  # optional
            'metric_type': MetricType.L2  # optional
        }

        status = milvus.create_collection(param)
        if not status.OK():
            print("Create collection failed: {}".format(status.message), file=sys.stderr)
            print("exiting ...", file=sys.stderr)
            sys.exit(1)

    # Show collections in Milvus server
    _, collections = milvus.list_collections()

    # Describe demo_collection
    _, collection = milvus.get_collection_info(collection_name)
    print(collection)

    # 10000 vectors with 16 dimension
    # element per dimension is float32 type
    # vectors should be a 2-D array
    vectors = [[random.random() for _ in range(_DIM)] for _ in range(100000)]

    # You can also use numpy to generate random vectors:
    #     `vectors = np.random.rand(10000, 16).astype(np.float32)`

    def _insert_callback(status, ids):
        if status.OK():
            print("Insert successfully")
        else:
            print("Insert failed.", status.message)

    # Insert vectors into demo_collection, adding callback function
    insert_future = milvus.insert(collection_name=collection_name, records=vectors, _async=True,
                                  _callback=_insert_callback)
    # Or invoke result() to get results:
    #   insert_future = milvus.insert(collection_name=collection_name, records=vectors, _async=True)
    #   status, ids = insert_future.result()
    insert_future.done()

    # Flush collection  inserted data to disk.
    def _flush_callback(status):
        if status.OK():
            print("Flush successfully")
        else:
            print("Flush failed.", status.message)

    flush_future = milvus.flush([collection_name], _async=True, _callback=_flush_callback)
    # Or invoke result() to get results:
    #   flush_future = milvus.flush([collection_name], _async=True)
    #   status = flush_future.result()
    flush_future.done()

    def _compact_callback(status):
        if status.OK():
            print("Compact successfully")
        else:
            print("Compact failed.", status.message)

    compact_furure = milvus.compact(collection_name, _async=True, _cakkback=_compact_callback)
    # Or invoke result() to get results:
    #   compact_future = milvus.compact(collection_name, _async=True)
    #   status = compact_future.result()
    compact_furure.done()

    # Get demo_collection row count
    status, result = milvus.count_entities(collection_name)

    # present collection info
    _, info = milvus.get_collection_stats(collection_name)
    print(info)

    # create index of vectors, search more rapidly
    index_param = {
        'nlist': 2048
    }

    def _index_callback(status):
        if status.OK():
            print("Create index successfully")
        else:
            print("Create index failed.", status.message)

    # Create ivflat index in demo_collection
    # You can search vectors without creating index. however, Creating index help to
    # search faster
    print("Creating index: {}".format(index_param))
    index_future = milvus.create_index(collection_name, IndexType.IVF_FLAT, index_param, _async=True,
                                       _callback=_index_callback)
    # Or invoke result() to get results:
    #   index_future = milvus.create_index(collection_name, IndexType.IVF_FLAT, index_param, _async=True)
    #   status = index_future.result()
    index_future.done()

    # describe index, get information of index
    status, index = milvus.get_index_info(collection_name)
    print(index)

    # Use the top 10 vectors for similarity search
    query_vectors = vectors[0:10]

    # execute vector similarity search
    search_param = {
        "nprobe": 16
    }

    print("Searching ... ")

    def _search_callback(status, results):
        # if status.OK():
        #     print("Search successfully")
        # else:
        #     print("Search failed.", status.message)
        if status.OK():
            # indicate search result
            # also use by:
            #   `results.distance_array[0][0] == 0.0 or results.id_array[0][0] == ids[0]`
            if results[0][0].distance == 0.0:  # or results[0][0].id == ids[0]:
                print('Query result is correct')
            else:
                print('Query result isn\'t correct')

            # print results
            print(results)
        else:
            print("Search failed. ", status)

    param = {
        'collection_name': collection_name,
        'query_records': query_vectors,
        'top_k': 1,
        'params': search_param,
        "_async": True,
        "_callback": _search_callback
    }
    search_future = milvus.search(**param)
    # Or invoke result() to get results:
    #
    #   param = {
    #       'collection_name': collection_name,
    #       'query_records': query_vectors,
    #       'top_k': 1,
    #       'params': search_param,
    #       "_async": True,
    #   }
    #   search_future = milvus.search(param)
    #   status, results = index_future.result()

    search_future.done()

    # Delete demo_collection
    status = milvus.drop_collection(collection_name)


if __name__ == '__main__':
    main()



# This program demos how to use milvus python client with multi-process
# See more details and caution in:
# https://milvus.io/docs/v0.6.0/faq/operational_faq.md#Why-does-my-multiprocessing-program-fail

import os
from multiprocessing import Process

from factorys import *
from milvus import Milvus

############### Global variable ###########
collection_name = 'test_test'

param = {'collection_name': collection_name,
         'dimension': 128,
         'index_file_size': 1024,
         'metric_type': MetricType.L2}

server_config = {
    'host': "127.0.0.1",
    'port': "19530"
}


## Utils
def _generate_vectors(_dim, _num):
    return [[random.random() for _ in range(_dim)] for _ in range(_num)]


def prepare_collection(_collection_name):
    def _create_collection(_collection_param):
        milvus = Milvus(**server_config)
        status, ok = milvus.has_collection(_collection_name)
        if ok:
            print("Collection {} found, now going to delete it".format(_collection_name))
            status = milvus.drop_collection(_collection_name)
            if not status.OK():
                raise Exception("Delete collection error")
            print("delete collection {} successfully!".format(_collection_name))
        time.sleep(5)

        status, ok = milvus.has_collection(_collection_name)
        if ok:
            raise Exception("Delete collection error")

        status = milvus.create_collection(param)
        if not status.OK():
            print("Create collection {} failed".format(_collection_name))
        milvus.close()

    # generate a process to run func `_create_collection`. A exception will be raised if
    # func `_create_collection` run in main process
    p = Process(target=_create_collection, args=(param,))
    p.start()
    p.join()


def multi_insert(_collection_name):
    # contain whole subprocess
    process_list = []

    def _add():
        milvus = Milvus(**server_config)

        vectors = _generate_vectors(128, 10000)
        print('\n\tPID: {}, insert {} vectors'.format(os.getpid(), 10000))
        status, _ = milvus.insert(_collection_name, vectors)
        if not status.OK():
            print("PID {} insert failed: {}".format(os.getpid(), status.message))
        milvus.close()

    for i in range(10):
        p = Process(target=_add)
        process_list.append(p)
        p.start()

    # block main process until whole sub process exit
    for p in process_list:
        p.join()

    print("insert vector successfully!")


def validate_insert(_collection_name):
    milvus = Milvus(**server_config)
    milvus.flush([_collection_name])
    status, count = milvus.count_entities(_collection_name)
    assert count == 10 * 10000, "Insert validate fail. Vectors num is not matched."

    # drop collcetion
    print("Drop collection ...")
    milvus.drop_collection(_collection_name)
    milvus.close()


def main(_collection_name):
    prepare_collection(_collection_name)

    # use multiple process to insert data
    multi_insert(_collection_name)

    # sleep 3 seconds to wait for vectors inserted Preservation
    time.sleep(3)

    validate_insert(_collection_name)


if __name__ == '__main__':
    main(collection_name)



import random
import time
import pytest
from milvus.client.utils import merge_results


@pytest.mark.skip
def test_merge_results(gcon):
    table_name = "merge_test"

    # delete table is exists
    status, ok = gcon.has_table(table_name)
    if ok:
        gcon.drop_table(table_name)

    table = {"table_name": table_name, "dimension": 128, "index_file_size": 100}
    gcon.create_table(table)

    vectors = [[random.random() for _ in range(128)] for _ in range(1000 * 1000)]

    gcon.insert(table_name, vectors)
    time.sleep(5)

    query_1_vectors = vectors[: 5]

    results_list = []
    for i in range(2000):
        status, results = gcon.search_in_files(table_name, top_k=5, nprobe=5, query_records=query_1_vectors,
                                               file_ids=[i])
        if status.OK():
            results_list.append(results.raw)

    topk_merge_result = merge_results(results_list, 5)

    status, result = gcon.search(table_name, top_k=5, nprobe=5, query_records=query_1_vectors)

    for merge_ids, r_ids in zip(topk_merge_result.id_array, result.id_array):
        for merge_id, r_id in zip(merge_ids, r_ids):
            assert merge_id == r_id

    gcon.drop_table(table_name)



import pytest

from milvus import ParamError


class TestSegment:
    def test_get_segment_ids(self, gcon, gvector):
        status, info = gcon.get_collection_stats(gvector)
        assert status.OK()
        seg0 = info["partitions"][0]["segments"][0]

        status, ids = gcon.list_id_in_segment(gvector, seg0["name"])
        assert status.OK(), status.message
        assert isinstance(ids, list)
        assert len(ids) == 10000

    @pytest.mark.parametrize("collection", [123, None, [], {}, "", True, False])
    @pytest.mark.parametrize("segment", [123, None, [], {}, "", True, False])
    def test_get_segment_invalid_param(self, collection, segment, gcon):
        with pytest.raises(ParamError):
            gcon.list_id_in_segment(collection, segment)

    def test_get_segment_non_existent_collection_segment(self, gcon, gcollection):
        status, _ = gcon.list_id_in_segment("ijojojononsfsfswgsw", "aaa")
        assert not status.OK()

        status, _ = gcon.list_id_in_segment(gcollection, "aaaaaa")
        assert not status.OK()



import pytest

from milvus import Milvus


@pytest.mark.skip
class TestPing:
    def test_ping_normal(self, gip):
        client = Milvus(*gip)
        assert client.ping()

    @pytest.mark.parametrize("ip", ["aa", 123, True])
    @pytest.mark.parametrize("port", ["bbb", False])
    def test_ping_invalid_addr(self, ip, port):
        with pytest.raises(Exception):
            client = Milvus(ip, port)
            client.ping()



import random


def gen_collection(connection, collection_name, dimension, metric_type, index_file_size=1024):
    collection_param = {
        "collection_name": collection_name,
        "dimension": dimension,
        "metric_type": metric_type,
        "index_file_size": index_file_size
    }

    return connection.create_collection(collection_param)


def insert_data(connection, collection_name, dim, num, partition_tag=None):
    vectors = [[random.random() for _ in range(dim)] for _ in range(num)]

    status, ids = connection.insert(collection_name, vectors, partition_tag=partition_tag)
    return status, ids, vectors



import random
import sys

from milvus import *

if __name__ == '__main__':
    collection_name_ = "test_collection_search_after_delete_11"

    client = Milvus(host="192.168.1.57", port=19531)
    client.create_collection({"collection_name": collection_name_, "dimension": 128})

    del_vecs = []
    del_ids = []
    for i in range(5):
        print(f"Insert round {i} ...")
        vectors = [[random.random() for _ in range(128)] for _ in range(100000)]
        del_vecs.append(vectors[0])
        vec_ids = [i * 100000 + j for j in range(100000)]
        del_ids.append(vec_ids[0])
        status, ids = client.insert(collection_name_, vectors, ids=vec_ids)
        if not status.OK():
            print(f"Insert failed: {status}", file=sys.stderr)
            sys.exit(1)

    client.flush([collection_name_])
    print("Create index ... ")
    status = client.create_index(collection_name_, IndexType.IVF_FLAT, params={"nlist": 1024})
    if not status.OK():
        print(f"Create index failed: {status}", file=sys.stderr)
        sys.exit(1)

    print("Search before delete ....")
    status, results = client.search(collection_name_, 5, del_vecs, params={"nprobe": 10})

    print("Delete entity ... ")
    status = client.delete_entity_by_id(collection_name_, id_array=del_ids)
    if not status.OK():
        print(f"Create index failed: {status}", file=sys.stderr)
        sys.exit(1)

    client.flush([collection_name_])
    print("Search after delete ... ")
    status, results = client.search(collection_name_, 5, del_vecs, params={"nprobe": 10})
    if not status.OK():
        print(f"Search failed: {status}", file=sys.stderr)
        sys.exit(1)

    for del_id, result in zip(del_ids, results):
        if result[0].id == del_id:
            print(f"Unexcepted search result. {result[0]}")

    client.drop_collection(collection_name_)



import pytest

from milvus import IndexType
from milvus import ParamError


class TestCreateIndex:
    def test_create_index_normal(self, gcon, gvector):
        status = gcon.create_index(gvector, IndexType.IVF_FLAT, {"nlist": 1024})
        assert status.OK()

    @pytest.mark.parametrize("index,param", [(IndexType.FLAT, {"nlist": 1024}),
                                             (IndexType.IVF_FLAT, {"nlist": 1024}),
                                             (IndexType.IVF_SQ8, {"nlist": 1024}),
                                             (IndexType.IVF_SQ8_H, {"nlist": 1024}),
                                             (IndexType.IVF_PQ, {"m": 16, "nlist": 1024}),
                                             (IndexType.HNSW, {"M": 16, "efConstruction": 500}),
                                             (IndexType.RNSG, {"search_length": 45, "out_degree": 50,
                                                               "candidate_pool_size": 300, "knng": 100}),
                                             (IndexType.ANNOY, {"n_trees": 20})])
    # @pytest.mark.parametrize("index,param", [(IndexType.IVF_SQ8_H, {"nlist": 1024})])
    def test_create_index_whole(self, index, param, gcon, gvector):
        status, mode = gcon._cmd("mode")
        assert status.OK()

        if mode == "GPU" and index in (IndexType.IVF_PQ,):
            pytest.skip("Index {} not support in GPU version".format(index))
        if mode == "CPU" and index in (IndexType.IVF_SQ8_H,):
            pytest.skip("Index {} not support in CPU version".format(index))

        status = gcon.create_index(gvector, index, param)
        assert status.OK()

    def test_create_index_async(self, gcon, gvector, ghandler):
        if ghandler == "HTTP":
            pytest.skip("HTTP handler not support async")
        future = gcon.create_index(gvector, IndexType.IVFLAT, params={"nlist": 1024}, _async=True)
        status = future.result()
        assert status.OK()

    def test_create_index_async_callback(self, gcon, gvector, ghandler):
        if ghandler == "HTTP":
            pytest.skip("HTTP handler not support async")

        def cb(status):
            assert status.OK()

        future = gcon.create_index(gvector, IndexType.IVFLAT, params={"nlist": 1024}, _async=True, _callback=cb)
        future.done()

    @pytest.mark.parametrize("index", [IndexType.INVALID, -1, 100, ""])
    def test_create_index_invalid_index(self, index, gcon, gvector):
        with pytest.raises(ParamError):
            gcon.create_index(gvector, index, {})

    def test_create_index_missing_index(self, gcon, gvector):
        status = gcon.create_index(gvector, params={"nlist": 1024})
        assert status.OK()


class TestDescribeIndex:
    def test_get_index_info_normal(self, gcon, gvector):
        status = gcon.create_index(gvector, IndexType.IVFLAT, params={"nlist": 1024})
        assert status.OK()

        status, index = gcon.get_index_info(gvector)
        assert status.OK()
        assert index.collection_name == gvector
        assert index.index_type == IndexType.IVFLAT
        assert index.params == {"nlist": 1024}

    @pytest.mark.parametrize("collection", [123, None, []])
    def test_get_index_info_invalid_name(self, collection, gcon):
        with pytest.raises(ParamError):
            gcon.get_index_info(collection)

    def test_get_index_info_non_existent(self, gcon):
        status, _ = gcon.get_index_info("non_existent")
        assert not status.OK()


class TestDropIndex:
    def test_drop_index_normal(self, gcon, gvector):
        status = gcon.create_index(gvector, IndexType.IVFLAT, {"nlist": 1024})
        assert status.OK()

    @pytest.mark.parametrize("collection", [123, None, []])
    def test_drop_index_invalid_name(self, collection, gcon):
        with pytest.raises(ParamError):
            gcon.drop_index(collection)

    def test_drop_index_non_existent(self, gcon):
        status = gcon.drop_index("non_existent")
        assert not status.OK()



# This program demos how to connect to Milvus vector database, 
# create a vector table, 
# insert 10 vectors, 
# and execute a vector similarity search.
import random
from milvus import Milvus, IndexType, MetricType
import time

# Milvus server IP address and port.
# You may need to change _HOST and _PORT accordingly.
_HOST = '127.0.0.1'
_PORT = '19530'  # default value

# Vector parameters
_DIM = 16  # dimension of vector
_INDEX_FILE_SIZE = 32  # max file size of stored index


def main():
    # Connect to Milvus server
    # You may need to change _HOST and _PORT accordingly
    param = {'host': _HOST, 'port': _PORT}

    # You can create a instance specified server addr and
    # invoke rpc method directly
    client = Milvus(**param)
    # Create collection demo_collection if it dosen't exist.
    collection_name = 'demo_partition_collection'
    partition_tag = "random"

    # create collection
    param = {
        'collection_name': collection_name,
        'dimension': _DIM,
        'index_file_size': _INDEX_FILE_SIZE,  # optional
        'metric_type': MetricType.L2  # optional
    }

    client.create_collection(param)

    # Show collections in Milvus server
    _, collections = client.list_collections()

    # Describe collection
    _, collection = client.get_collection_info(collection_name)
    print(collection)

    # create partition
    client.create_partition(collection_name, partition_tag=partition_tag)
    # display partitions
    _, partitions = client.list_partitions(collection_name)

    # 10000 vectors with 16 dimension
    # element per dimension is float32 type
    # vectors should be a 2-D array
    vectors = [[random.random() for _ in range(_DIM)] for _ in range(10000)]
    # You can also use numpy to generate random vectors:
    #     `vectors = np.random.rand(10000, 16).astype(np.float32).tolist()`

    # Insert vectors into partition of collection, return status and vectors id list
    status, ids = client.insert(collection_name=collection_name, records=vectors, partition_tag=partition_tag)

    # Wait for 6 seconds, until Milvus server persist vector data.
    time.sleep(6)

    # Get demo_collection row count
    status, num = client.count_entities(collection_name)

    # create index of vectors, search more rapidly
    index_param = {
        'nlist': 2048
    }

    # Create ivflat index in demo_collection
    # You can search vectors without creating index. however, Creating index help to
    # search faster
    status = client.create_index(collection_name, IndexType.IVF_FLAT, index_param)

    # describe index, get information of index
    status, index = client.get_index_info(collection_name)
    print(index)

    # Use the top 10 vectors for similarity search
    query_vectors = vectors[0:10]

    # execute vector similarity search, search range in partition `partition1`
    search_param = {
        "nprobe": 10
    }

    param = {
        'collection_name': collection_name,
        'query_records': query_vectors,
        'top_k': 1,
        'partition_tags': ["random"],
        'params': search_param
    }
    status, results = client.search(**param)

    if status.OK():
        # indicate search result
        # also use by:
        #   `results.distance_array[0][0] == 0.0 or results.id_array[0][0] == ids[0]`
        if results[0][0].distance == 0.0 or results[0][0].id == ids[0]:
            print('Query result is correct')
        else:
            print('Query result isn\'t correct')

    # print results
    print(results)

    # Drop partition. You can also invoke `drop_collection()`, so that all of partitions belongs to
    # designated collections will be deleted.
    status = client.drop_partition(collection_name, partition_tag)

    # Delete collection. All of partitions of this collection will be dropped.
    status = client.drop_collection(collection_name)


if __name__ == '__main__':
    main()



import random

import pytest

from milvus import ParamError
from factorys import records_factory

dim = 128
nq = 100


class TestFlush:
    def test_flush(self, gcon):
        collection_param = {
            "collection_name": '',
            "dimension": dim
        }

        collection_list = ["test_flush_1", "test_flush_2", "test_flush_3"]
        vectors = records_factory(dim, nq)
        for collection in collection_list:
            collection_param["collection_name"] = collection

            gcon.create_collection(collection_param)

            gcon.insert(collection, vectors)

        status = gcon.flush(collection_list)
        assert status.OK()

        for collection in collection_list:
            gcon.drop_collection(collection)

    def test_flush_with_none(self, gcon, gcollection):
        collection_param = {
            "collection_name": '',
            "dimension": dim
        }

        collection_list = ["test_flush_1", "test_flush_2", "test_flush_3"]
        vectors = records_factory(dim, nq)
        for collection in collection_list:
            collection_param["collection_name"] = collection

            gcon.create_collection(collection_param)

            gcon.insert(collection, vectors)

        status = gcon.flush()
        assert status.OK(), status.message

        for collection in collection_list:
            gcon.drop_collection(collection)

    def test_flush_async_normal(self, gcon, gcollection, ghandler):
        if ghandler == "HTTP":
            pytest.skip("HTTP handler not support async")

        records = records_factory(dim, nq)
        gcon.insert(gcollection, records)
        future = gcon.flush([gcollection], _async=True)
        status = future.result()
        assert status.OK()

    def test_flush_async_callback(self, gcon, gcollection, ghandler):
        if ghandler == "HTTP":
            pytest.skip("HTTP handler not support async")

        def cb(status):
            assert status.OK()

        records = records_factory(dim, nq)
        gcon.insert(gcollection, records)
        future = gcon.flush([gcollection], _async=True, _callback=cb)
        future.done()


class TestCompact:
    def test_compact_normal(self, gcon, gcollection):
        vectors = [[random.random() for _ in range(128)] for _ in range(10000)]
        status, ids = gcon.insert(collection_name=gcollection, records=vectors)
        assert status.OK()

        status = gcon.compact(gcollection)
        assert status.OK(), status.message

    def test_compact_after_delete(self, gcon, gcollection):
        vectors = [[random.random() for _ in range(128)] for _ in range(10000)]
        status, ids = gcon.insert(collection_name=gcollection, records=vectors)
        assert status.OK(), status.message

        status = gcon.flush([gcollection])
        assert status.OK(), status.message

        status = gcon.delete_entity_by_id(gcollection, ids[100:1000])
        assert status, status.message

        status = gcon.compact(gcollection)
        assert status.OK(), status.message

    def test_compact_async_normal(self, gcon, gvector, ghandler):
        if ghandler == "HTTP":
            pytest.skip("HTTP handler not support async")

        future = gcon.compact(gvector, _async=True)
        status = future.result()
        assert status.OK()

    def test_compact_async_callback(self, gcon, gvector, ghandler):
        if ghandler == "HTTP":
            pytest.skip("HTTP handler not support async")

        def cb(status):
            assert status.OK()

        future = gcon.compact(gvector, _async=True, _callback=cb)
        future.done()

    def test_compact_with_empty_collection(self, gcon, gcollection):
        status = gcon.compact(gcollection)
        assert status.OK(), status.message

    def test_compact_with_non_exist_name(self, gcon):
        status = gcon.compact(collection_name="die333")
        assert not status.OK()

    @pytest.mark.parametrize("name", [123, None, [123], True, {}])
    def test_compact_with_invalid_name(self, name, gcon):
        with pytest.raises(ParamError):
            gcon.compact(collection_name=name)


class TestCmd:
    def test_cmd_version(self, gcon):
        status, reply = gcon.server_version()
        assert status.OK()

    @pytest.mark.parametrize("cmd", ["version", "status", "mode", "tasktable", "get_system_info", "build_commit_id"])
    def test_cmd_whole(self, cmd, gcon):
        status, reply = gcon._cmd(cmd)
        assert status.OK()


class TestConfig:
    def test_get_config(self, gcon):
        status, value = gcon.get_config("cache", "cache_size")
        assert status.OK()

    def test_set_config(self, gcon):
        status, value = gcon.get_config("cache", "cache_size")
        assert status.OK()

        status, reply = gcon.set_config("cache", "cache_size", value)
        assert status.OK()




import sys
sys.path.append(".")
from unittest import mock

import grpc
import pytest
from grpc._channel import _UnaryUnaryMultiCallable as Uum

from milvus import Milvus, MetricType, IndexType, ParamError

from factorys import collection_schema_factory

from utils import MockGrpcError


class TestCreateCollection:
    def test_create_collection_normal(self, gcon):
        collection_param = collection_schema_factory()
        status = gcon.create_collection(collection_param)
        assert status.OK(), status.message

        status = gcon.drop_collection(collection_param["collection_name"])
        assert status.OK(), status.message

    def test_create_collection_repeat(self, gcon):
        collection_param = collection_schema_factory()
        status = gcon.create_collection(collection_param)
        assert status.OK(), status.message

        status = gcon.create_collection(collection_param)
        assert not status.OK()

        status = gcon.drop_collection(collection_param["collection_name"])
        assert status.OK(), status.message

    @pytest.mark.parametrize("metric",
                             [MetricType.L2, MetricType.IP, MetricType.HAMMING,
                              MetricType.JACCARD, MetricType.TANIMOTO,
                              MetricType.SUPERSTRUCTURE, MetricType.SUBSTRUCTURE])
    def test_create_collection_whole_metric(self, metric, gcon):
        collection_param = {
            "collection_name": "test_create_collection_metric_" + str(metric),
            "dimension": 128,
            "metric_type": metric
        }

        status = gcon.create_collection(collection_param)
        assert status.OK()
        status = gcon.drop_collection(collection_param["collection_name"])
        assert status.OK(), status.message

    def test_create_collection_default_index_file_size(self, gcon):
        collection_param = {
            "collection_name": "test_create_collection_default_index_file_size",
            "dimension": 128,
            "metric_type": MetricType.L2
        }
        status = gcon.create_collection(collection_param)
        assert status.OK(), status.message

        status = gcon.drop_collection(collection_param["collection_name"])
        assert status.OK(), status.message

    def test_create_collection_default_metric_type(self, gcon):
        collection_param = {
            "collection_name": "test_create_collection_default_metric_type",
            "dimension": 128,
            "index_file_size": 100
        }
        status = gcon.create_collection(collection_param)
        assert status.OK(), status.message

        status = gcon.drop_collection(collection_param["collection_name"])
        assert status.OK(), status.message

    @pytest.mark.parametrize("metric", ["123", None, MetricType.INVALID, -1, 1000])
    def test_create_collection_invalid_metric(self, metric, gcon):
        collection_param = {
            "collection_name": "test_create_collection_invalid_metric_" + str(metric),
            "dimension": 128,
            "metric_type": metric
        }
        with pytest.raises(ParamError):
            gcon.create_collection(collection_param)

    @pytest.mark.parametrize("dim", ["123", None, [1]])
    def test_create_collection_invalid_dim(self, dim, gcon):
        collection_param = {
            "collection_name": "test_create_collection_invalid_dim_" + str(dim),
            "dimension": dim,
            "metric_type": MetricType.L2
        }
        with pytest.raises(ParamError):
            gcon.create_collection(collection_param)

    @pytest.mark.parametrize("index_file_size", ["123", None, [1]])
    def test_create_collection_invalid_index_file_size(self, index_file_size, gcon):
        collection_param = {
            "collection_name": "test_create_collection_invalid_index_file_size_" + str(index_file_size),
            "dimension": 128,
            "metric_type": MetricType.L2,
            "index_file_size": index_file_size
        }
        with pytest.raises(ParamError):
            gcon.create_collection(collection_param)

    def test_create_collection_exception(self, gcon):
        collection_param = {
            "collection_name": "test_create_collection_exceptions",
            "dimension": 128,
            "metric_type": MetricType.L2,
            "index_file_size": 10
        }

        mock_grpc_timeout = mock.MagicMock(side_effect=grpc.FutureTimeoutError())
        with mock.patch.object(Uum, 'future', mock_grpc_timeout):
            status = gcon.create_collection(collection_param)
            assert not status.OK()

        mock_grpc_error = mock.MagicMock(side_effect=MockGrpcError())
        with mock.patch.object(Uum, 'future', mock_grpc_error):
            status = gcon.create_collection(collection_param)
            assert not status.OK()

        with pytest.raises(Exception):
            mock_exception = mock.MagicMock(side_effect=Exception("error"))
            with mock.patch.object(Uum, 'future', mock_exception):
                status = gcon.create_collection(collection_param)
                assert not status.OK()


class TestHasCollection:
    def test_has_collection_normal(self, gcon, gcollection):
        status, ok = gcon.has_collection(gcollection)
        assert status.OK()
        assert ok

    @pytest.mark.parametrize("collection", [None, 123, [123], {}, ""])
    def test_has_collection_invalid_name(self, collection, gcon):
        with pytest.raises(ParamError):
            gcon.has_collection(collection)

    def test_has_collection_non_existent(self, gcon):
        status, ok = gcon.has_collection("sfsfsfsfsfsfsfsfsfsf")
        assert not (status.OK() and ok)


class TestDescribeCollection:
    def test_get_collection_info_normal(self, gcon, gcollection):
        status, info = gcon.get_collection_info(gcollection)
        assert status.OK()
        assert info.dimension == 128

    @pytest.mark.parametrize("collection", [None, 123, [123], {}])
    def test_get_collection_info_with_invalid_collection(self, collection, gcon):
        with pytest.raises(ParamError):
            gcon.get_collection_info(collection)

    def test_get_collection_info_non_existent(self, gcon):
        status, _ = gcon.get_collection_info("sfsfsfsfsfsfsfsfsfsf")
        assert not status.OK()


class TestCountCollection:
    def test_count_entities_normal(self, gcon, gvector):
        status, count = gcon.count_entities(gvector)
        assert status.OK()
        assert count == 10000

    @pytest.mark.parametrize("collection", [None, 123, [123], {}])
    def test_count_entities_invalid_name(self, collection, gcon):
        with pytest.raises(ParamError):
            gcon.count_entities(collection)

    def test_count_entities_non_existent(self, gcon):
        status, _ = gcon.count_entities("sfsfsfsfsfsfsfsfsfsf")
        assert not status.OK()


class TestCollectinStats:
    def test_get_collection_stats_normal(self, gcon, gvector):
        status, info = gcon.get_collection_stats(gvector)
        assert status.OK()
        assert info["row_count"] == 10000

        par0_stat = info["partitions"][0]
        assert par0_stat["tag"] == "_default"
        assert par0_stat["row_count"] == 10000

    @pytest.mark.parametrize("collection", [None, 123, [123], {}])
    def test_get_collection_stats_invalid_name(self, collection, gcon):
        with pytest.raises(ParamError):
            gcon.get_collection_stats(collection)

    def test_count_entities_non_existent(self, gcon):
        status, _ = gcon.get_collection_stats("sfsfsfsfsfsfsfsfsfsf")
        assert not status.OK()


class TestShowCollections:
    def test_list_collections(self, gcon, gcollection):
        status, names = gcon.list_collections()
        assert status.OK()
        assert gcollection in names


class TestDropCollection:
    def test_drop_collection_normal(self, gcon):
        status = gcon.create_collection({"collection_name": "test01", "dimension": 128})
        assert status.OK()

        status = gcon.drop_collection("test01")
        assert status.OK()

    @pytest.mark.parametrize("collection", [[], None, 123, {}])
    def test_drop_collection_invalid_name(self, collection, gcon):
        with pytest.raises(ParamError):
            gcon.drop_collection(collection)

    def test_drop_collection_non_existent(self, gcon):
        status = gcon.drop_collection("non_existent")
        assert not status.OK()


class TestLoadCollection:
    def test_load_collection_normal(self, gcon, gvector):
        status = gcon.load_collection(gvector)
        assert status.OK()

    @pytest.mark.parametrize("name", [[], bytes(), 123, True, False])
    def test_load_collection_invalid_name(self, name, gcon):
        with pytest.raises(ParamError):
            gcon.load_collection(name)

    def test_load_collection_non_existent(self, gcon):
        status = gcon.load_collection("test_load_collection_non_existent")
        assert not status.OK()

    def test_load_collection_with_partition(self, gcon, gvector):
        status = gcon.load_collection(gvector, ["_default"])
        assert status.OK()

    def test_load_collection_with_partition_non_exist(self, gcon, gvector):
        status = gcon.load_collection(gvector, ["_default", "AQAA"])
        assert status.OK()

    def test_load_collection_with_all_partition_non_exist(self, gcon, gvector):
        status = gcon.load_collection(gvector, ["AQAA"])
        assert not status.OK()

class TestReleaseCollection:
    def test_release_collection_normal(self, gcon, gvector):
        status = gcon.load_collection(gvector)
        assert status.OK()
        status = gcon.release_collection(gvector)
        assert status.OK()

    @pytest.mark.parametrize("name", [[], bytes(), 123, True, False])
    def test_release_collection_invalid_name(self, name, gcon):
        with pytest.raises(ParamError):
            gcon.release_collection(name)

    def test_release_collection_non_existent(self, gcon):
        status = gcon.release_collection("test_release_collection_non_existent")
        assert not status.OK()

    def test_release_collection_with_partition(self, gcon, gvector):
        status = gcon.load_collection(gvector, ["_default"])
        assert status.OK()
        status = gcon.release_collection(gvector, ["_default"])
        assert status.OK()

    def test_release_collection_with_partition_non_exist(self, gcon, gvector):
        status = gcon.load_collection(gvector, ["_default"])
        assert status.OK()
        status = gcon.release_collection(gvector, ["_default", "test_release_collection_with_partition_non_exist"])
        assert status.OK()

    def test_release_collection_with_all_partition_non_exist(self, gcon, gvector):
        status = gcon.load_collection(gvector, ["_default"])
        assert status.OK()
        status = gcon.release_collection(gvector, ["test_release_collection_with_partition_non_exist"])
        assert not status.OK()



# This python shell demonstrate how to create IVF_PQ index
# and search
# Note that index IVF_PQ is not supported on server of GPU version,
# please make sure milvus server you used is on CPU version.

import random

from milvus import Milvus, IndexType, MetricType

from utils import gen_collection, insert_data

if __name__ == '__main__':
    collection_name = "milvus_demo_ivf_pq"

    client = Milvus()

    # create collection: dimension is 128, metric type is L2
    gen_collection(client, collection_name, 128, MetricType.L2)

    # insert 10000 vectors into collection
    _, _, vectors = insert_data(client, collection_name, 128, 10000)

    # flush data into disk for persistent storage
    client.flush([collection_name])

    # specify index param
    index_param = {
        "m": 16,
        "nlist": 1024
    }

    # create `IVF_PQ` index
    status = client.create_index(collection_name, IndexType.IVF_PQ, index_param)
    if status.OK():
        print("Create index IVF_PQ successfully\n")
    else:
        print("Create index fail: ", status)

    # select top 10 vectors from inserted as query vectors
    query_vectors = vectors[:10]

    # specify search param
    search_param = {
        "nprobe": 10
    }

    # specify topk is 1, search approximate nearest 1 neighbor
    status, result = client.search(collection_name, 2, query_vectors, params=search_param)

    if status.OK():
        # show search result
        print("Search successfully. Result:\n", result)
    else:
        print("Search fail")

    # drop collection
    client.drop_collection(collection_name)

    client.close()



import time

# Third party imports
import pytest

# local application imports
from factorys import gen_unique_str, fake, records_factory, bin_records_factory
from milvus import Milvus, IndexType, MetricType

default_host = "127.0.0.1"
default_grpc_port = 19530
default_http_port = 19121


def pytest_addoption(parser):
    parser.addoption("--ip", action="store", default=default_host)
    parser.addoption("--handler", action="store", default="GRPC")
    parser.addoption("--port", action="store", default=default_grpc_port)


@pytest.fixture(scope="module")
def gip(request):
    ip_ = request.config.getoption("--ip")
    port_ = request.config.getoption("--port")

    return ip_, port_


@pytest.fixture(scope="module")
def ghandler(request):
    handler_ = request.config.getoption("--handler")
    return handler_


@pytest.fixture(scope="module")
def connect(request, ghandler):
    ip = request.config.getoption("--ip")
    handler = request.config.getoption("--handler")
    port_default = default_http_port if handler == "HTTP" else default_grpc_port
    port = request.config.getoption("--port", default=port_default)

    client = Milvus(host=ip, port=port, handler=ghandler)
    # milvus.connect()

    def fin():
        try:
            client.close()
        except:
            pass

    request.addfinalizer(fin)
    return client


@pytest.fixture(scope="module")
def gcon(request, ghandler):
    ip = request.config.getoption("--ip")
    port = request.config.getoption("--port")
    client = Milvus(host=ip, port=port, handler=ghandler)

    def fin():
        try:
            client.close()
            pass
        except Exception as e:
            print(e)
            pass

    request.addfinalizer(fin)
    return client


@pytest.fixture(scope="module")
def args(request):
    ip = request.config.getoption("--ip")
    port = request.config.getoption("--port")
    args = {"ip": ip, "port": port}
    return args


@pytest.fixture(scope="function")
def table(request, connect):
    ori_table_name = getattr(request.module, "table_id", "test")
    table_name = gen_unique_str(ori_table_name)
    dim = getattr(request.module, "dim", "128")
    param = {'collection_name': table_name,
             'dimension': dim,
             'index_type': IndexType.IVFLAT,
             'metric_type': MetricType.L2
             }
    connect.create_collection(param)

    def teardown():
        connect.drop_collection(table_name)

    request.addfinalizer(teardown)

    return table_name


@pytest.fixture(scope="function")
def gcollection(request, gcon):
    collection_name = fake.collection_name()
    dim = getattr(request.module, "dim", 128)

    param = {'collection_name': collection_name,
             'dimension': dim,
             'index_file_size': 1024,
             'metric_type': MetricType.L2
             }
    gcon.create_collection(param)
    gcon.flush([collection_name])

    def teardown():
        status, collection_names = gcon.list_collections()
        for name in collection_names:
            gcon.drop_collection(name)

    request.addfinalizer(teardown)

    return collection_name


@pytest.fixture(scope='function')
def gvector(request, gcon, gcollection):
    dim = getattr(request.module, 'dim', 128)

    records = records_factory(dim, 10000)

    gcon.insert(gcollection, records)
    gcon.flush([gcollection])

    return gcollection


@pytest.fixture(scope="function")
def gbcollection(request, gcon):
    collection_name = fake.collection_name()
    dim = getattr(request.module, "dim", 128)

    param = {'collection_name': collection_name,
             'dimension': dim,
             'index_file_size': 1024,
             'metric_type': MetricType.HAMMING
             }
    gcon.create_collection(param)
    gcon.flush([collection_name])

    def teardown():
        status, collection_names = gcon.list_collections()
        for name in collection_names:
            gcon.drop_collection(name)

    request.addfinalizer(teardown)

    return collection_name


@pytest.fixture(scope='function')
def gbvector(request, gcon, gbcollection):
    dim = getattr(request.module, 'dim', 128)

    records = bin_records_factory(dim, 10000)

    gcon.insert(gbcollection, records)
    gcon.flush([gbcollection])

    return gbcollection



# This program demos how to connect to Milvus vector database, 
# create a vector collection,
# insert 10 vectors, 
# and execute a vector similarity search.

import random

from milvus import Milvus, IndexType, MetricType, Status

# Milvus server IP address and port.
# You may need to change _HOST and _PORT accordingly.
_HOST = '127.0.0.1'
_PORT = '19530'  # default value
# _PORT = '19121'  # default http value

# Vector parameters
_DIM = 8  # dimension of vector

_INDEX_FILE_SIZE = 32  # max file size of stored index


def main():
    # Specify server addr when create milvus client instance
    # milvus client instance maintain a connection pool, param
    # `pool_size` specify the max connection num.
    milvus = Milvus(_HOST, _PORT)

    # Create collection demo_collection if it dosen't exist.
    collection_name = 'example_collection_'

    status, ok = milvus.has_collection(collection_name)
    if not ok:
        param = {
            'collection_name': collection_name,
            'dimension': _DIM,
            'index_file_size': _INDEX_FILE_SIZE,  # optional
            'metric_type': MetricType.L2  # optional
        }

        milvus.create_collection(param)

    # Show collections in Milvus server
    _, collections = milvus.list_collections()

    # Describe demo_collection
    _, collection = milvus.get_collection_info(collection_name)
    print(collection)

    # 10000 vectors with 128 dimension
    # element per dimension is float32 type
    # vectors should be a 2-D array
    vectors = [[random.random() for _ in range(_DIM)] for _ in range(10)]
    print(vectors)
    # You can also use numpy to generate random vectors:
    #   vectors = np.random.rand(10000, _DIM).astype(np.float32)

    # Insert vectors into demo_collection, return status and vectors id list
    status, ids = milvus.insert(collection_name=collection_name, records=vectors)
    if not status.OK():
        print("Insert failed: {}".format(status))

    # Flush collection  inserted data to disk.
    milvus.flush([collection_name])
    # Get demo_collection row count
    status, result = milvus.count_entities(collection_name)

    # present collection statistics info
    _, info = milvus.get_collection_stats(collection_name)
    print(info)

    # Obtain raw vectors by providing vector ids
    status, result_vectors = milvus.get_entity_by_id(collection_name, ids[:10])

    # create index of vectors, search more rapidly
    index_param = {
        'nlist': 2048
    }

    # Create ivflat index in demo_collection
    # You can search vectors without creating index. however, Creating index help to
    # search faster
    print("Creating index: {}".format(index_param))
    status = milvus.create_index(collection_name, IndexType.IVF_FLAT, index_param)

    # describe index, get information of index
    status, index = milvus.get_index_info(collection_name)
    print(index)

    # Use the top 10 vectors for similarity search
    query_vectors = vectors[0:10]

    # execute vector similarity search
    search_param = {
        "nprobe": 16
    }

    print("Searching ... ")

    param = {
        'collection_name': collection_name,
        'query_records': query_vectors,
        'top_k': 1,
        'params': search_param,
    }

    status, results = milvus.search(**param)
    if status.OK():
        # indicate search result
        # also use by:
        #   `results.distance_array[0][0] == 0.0 or results.id_array[0][0] == ids[0]`
        if results[0][0].distance == 0.0 or results[0][0].id == ids[0]:
            print('Query result is correct')
        else:
            print('Query result isn\'t correct')

        # print results
        print(results)
    else:
        print("Search failed. ", status)

    # Delete demo_collection
    status = milvus.drop_collection(collection_name)


if __name__ == '__main__':
    main()



import random
import sys

from milvus import Milvus, MetricType, IndexType

_DIM = 128

if __name__ == '__main__':
    client = Milvus(host="localhost", port=19121, handler="HTTP")

    collection_name = 'example_collection_vector'

    param = {
        'collection_name': collection_name,
        'dimension': _DIM,
        'index_file_size': 10,  # optional
        'metric_type': MetricType.L2  # optional
    }

    client.create_collection(param)

    # randomly generate 100000 vectors and insert collection
    vectors = [[random.random() for _ in range(_DIM)] for _ in range(10000)]
    client.insert(collection_name, vectors)

    # flush data to disk
    client.flush([collection_name])

    # query collection's statistical information.
    status, info = client.get_collection_stats(collection_name)
    if not status.OK():
        print("Query collection statistical information fail. exiting ....")
        sys.exit(1)

    # show collection information
    print("Total amount of vectors in collection {} is {}".format(collection_name, info["row_count"]))
    for par in info["partitions"]:
        print("\tpartition tag: {}, vector count: {}".format(par["tag"], par["row_count"]))
        # show segment information
        for seg in par["segments"]:
            print("\t\tsegment name: {}, vector count: {}, index: {}, storage size {:.3f} MB"
                  .format(seg["name"], seg["row_count"], seg["index_name"], seg["data_size"] / 1024 / 1024))

    # obtain vector ids from segment, then
    # get vector by specifying vector id
    segment0 = info["partitions"][0]["segments"][0]
    status, ids = client.list_id_in_segment(collection_name, segment0["name"])
    if not status.OK():
        print("Cannot obtain vector ids from segment {}. exiting ....".format(segment0["name"]))
        sys.exit(1)

    # obtain top 5 vector
    status, vectors = client.get_entity_by_id(collection_name, ids[:5])
    if not status.OK():
        print("Cannot obtain vector. exiting ....")
        sys.exit(1)

    # delete top 10 vectors
    status = client.delete_entity_by_id(collection_name, ids[:10])
    if status.OK():
        print("Delete top 10 vectors successfully")
    else:
        print("Error occurred when try to delete top 10 vectors. Reason: ", status.message)

    client.drop_collection(collection_name)




"""client module"""
from pkg_resources import get_distribution, DistributionNotFound

__version__ = '0.0.0.dev'

try:
    __version__ = get_distribution('pymilvus').version
except DistributionNotFound:
    # package is not installed
    pass



import threading

import pytest

from milvus import Milvus, NotConnectError, VersionError
from milvus.client.pool import ConnectionPool


class TestPool:
    def test_pool_max_conn(self, gip):
        ip_, port_ = gip
        pool = ConnectionPool(uri=f"tcp://{ip_}:{port_}", pool_size=10)

        def run(_pool):
            conn = _pool.fetch()
            assert conn.conn_id() < 10
            conn.has_collection("test_pool")

        thread_list = []
        for _ in range(10 * 3):
            thread = threading.Thread(target=run, args=(pool,))
            thread.start()
            thread_list.append(thread)

    def test_pool_from_stub(self, gip):
        ip_, port_ = gip
        client = Milvus(uri=f"tcp://{ip_}:{port_}", pool_size=10)

        def run(_client):
            _client.has_collection("test_pool")

        thread_list = []
        for _ in range(10 * 3):
            thread = threading.Thread(target=run, args=(client,))
            thread.start()
            thread_list.append(thread)

    @pytest.mark.skip
    def test_pool_args(self):
        with pytest.raises(NotConnectError):
            ConnectionPool(uri="tcp://123.456.780.0:9999", pool_size=10, try_connect=True)

        with pytest.raises(NotConnectError):
            ConnectionPool(uri="tcp://123.456.780.0:9999", pool_size=10, try_connect=False)



import pytest
import random

from milvus import ParamError
from factorys import records_factory

dim = 128
nq = 100


class TestSearch:
    topk = random.randint(1, 10)
    query_records = records_factory(dim, nq)
    search_param = {
        "nprobe": 10
    }

    def test_search_normal(self, gcon, gvector):
        param = {
            'collection_name': gvector,
            'query_records': self.query_records,
            'top_k': self.topk,
            'params': self.search_param
        }
        res, results = gcon.search(**param)
        assert res.OK()
        assert len(results) == nq
        for r in results:
            assert len(r) == self.topk

        assert results.shape[0] == nq
        assert results.shape[1] == self.topk

    def test_search_default_partition(self, gcon, gvector):
        param = {
            'collection_name': gvector,
            'query_records': self.query_records,
            'top_k': self.topk,
            'partition_tags': ["_default"],
            'params': self.search_param
        }
        res, results = gcon.search(**param)
        assert res.OK()
        assert len(results) == nq
        assert len(results[0]) == self.topk

        assert results.shape[0] == nq
        assert results.shape[1] == self.topk

    def test_search_result_contain__1(self, gcon, gcollection):
        vectors = [[random.random() for _ in range(128)] for _ in range(5)]
        status, _ = gcon.insert(gcollection, vectors)
        assert status.OK()

        status = gcon.flush([gcollection])
        assert status.OK()

        param = {
            'collection_name': gcollection,
            'query_records': self.query_records[:2],
            'top_k': 10,
            'params': self.search_param
        }
        res, results = gcon.search(**param)
        assert res.OK()
        assert len(results) == 2
        assert len(results[0]) == 5

        assert results.shape[0] == 2
        # assert results.shape[1] == self.topk

    def test_search_async_normal(self, gcon, gvector, ghandler):
        if ghandler == "HTTP":
            pytest.skip("HTTP handler not support async")

        param = {
            'collection_name': gvector,
            'query_records': self.query_records,
            'top_k': self.topk,
            'params': self.search_param,
            '_async': True
        }
        future = gcon.search(**param)
        status, results = future.result()

        assert status.OK()
        assert len(results) == nq
        assert len(results[0]) == self.topk

        assert results.shape[0] == nq
        assert results.shape[1] == self.topk

    def test_search_async_callback(self, gcon, gvector, ghandler):
        if ghandler == "HTTP":
            pytest.skip("HTTP handler not support async")

        param = {
            'collection_name': gvector,
            'query_records': self.query_records,
            'top_k': self.topk,
            'params': self.search_param,
            '_async': True
        }

        def cb(status, results):
            assert status.OK()
            assert len(results) == nq
            assert len(results[0]) == self.topk

            assert results.shape[0] == nq
            assert results.shape[1] == self.topk

        future = gcon.search(_callback=cb, **param)
        future.done()

    @pytest.mark.parametrize("query_records", [[], None, "", 123])
    def test_search_invalid_query_records(self, query_records, gcon, gvector):
        param = {
            'collection_name': gvector,
            'query_records': query_records,
            'top_k': self.topk,
            'params': self.search_param,
        }
        with pytest.raises(ParamError):
            gcon.search(**param)

    @pytest.mark.parametrize("params", [[], "", 123])
    def test_search_invalid_params(self, params, gcon, gvector):
        param = {
            'collection_name': gvector,
            'query_records': self.query_records,
            'top_k': self.topk,
            'params': params,
        }
        with pytest.raises(ParamError):
            gcon.search(**param)

    @pytest.mark.parametrize("topk", [[], "", None, {}])
    def test_search_invalid_topk(self, topk, gcon, gvector):
        param = {
            'collection_name': gvector,
            'query_records': self.query_records,
            'top_k': topk,
            'params': self.search_param,
        }
        with pytest.raises(ParamError):
            gcon.search(**param)


@pytest.mark.skip(reason="departed")
class TestSearchByIds:
    vectors = [[random.random() for _ in range(dim)] for _ in range(nq)]

    def test_search_by_ids_normal(self, gcon, gcollection):
        status, ids = gcon.insert(gcollection, self.vectors)
        assert status.OK()

        status = gcon.flush([gcollection])
        assert status.OK()

        search_param = {
            "nprobe": 10
        }
        status, results = gcon.search_by_ids(gcollection, ids[0: 10], 10, params=search_param)
        assert status.OK()
        # assert len(results) == 10
        # assert len(results[0]) == 10
        #
        # assert results.shape[0] == 10
        # assert results.shape[1] == 10

    def test_search_by_ids_with_empty_param(self, gcon, gcollection):
        status, _ = gcon.search_by_ids(gcollection, [1], 1, params=None)
        assert not status.OK()

        status, _ = gcon.search_by_ids(gcollection, [1], 1, params={})
        assert not status.OK()

    @pytest.mark.parametrize("ids", [None, "123", False])
    def test_seach_by_ids_with_invalid_ids(self, ids, gcon, gcollection):
        with pytest.raises(ParamError):
            gcon.search_by_ids(gcollection, ids, 1, params={"nprobe": 10})

    @pytest.mark.parametrize("collection", [None, [], {}, False])
    def test_search_by_ids_with_invalid_collection(self, collection, gcon):
        with pytest.raises(ParamError):
            gcon.search_by_ids(collection, [1], 1, params={"nprobe": 10})

    @pytest.mark.parametrize("topk", [[], "", None, {}])
    def test_search_by_ids_invalid_topk(self, topk, gcon, gcollection):
        with pytest.raises(ParamError):
            gcon.search(gcollection, [1], topk, params={"nprobe": 10})

    @pytest.mark.parametrize("params", [[1, 2, 3], "test", True, 128])
    def test_search_by_ids_invalid_params(self, params, gcon, gcollection):
        with pytest.raises(ParamError):
            gcon.search(gcollection, [1], 1, params=params)


class TestSearchInFiles:
    def test_search_in_segment_normal(self, gcon, gvector):
        search_param = {
            "nprobe": 10
        }

        query_vectors = [[random.random() for _ in range(128)] for _ in range(100)]
        for i in range(5000):
            status, _ = gcon.search_in_segment(gvector, file_ids=[i], top_k=1,
                                             query_records=query_vectors, params=search_param)
            if status.OK():
                return
        assert False

    def test_search_in_segment_async(self, gcon, gvector, ghandler):
        if ghandler == "HTTP":
            pytest.skip("HTTP handler not support async")

        search_param = {
            "nprobe": 10
        }

        query_vectors = [[random.random() for _ in range(128)] for _ in range(100)]
        for i in range(5000):
            future = gcon.search_in_segment(gvector, file_ids=[i], top_k=1, query_records=query_vectors,
                                          params=search_param, _async=True)
            status, _ = future.result()
            if status.OK():
                return
        assert False

    def test_search_in_segment_async_callback(self, gcon, gvector, ghandler):
        if ghandler == "HTTP":
            pytest.skip("HTTP handler not support async")

        def cb(status, results):
            print("Search status: ", status)

        search_param = {
            "nprobe": 10
        }

        query_vectors = [[random.random() for _ in range(128)] for _ in range(100)]
        for i in range(5000):
            future = gcon.search_in_segment(gvector, file_ids=[i], top_k=1, query_records=query_vectors,
                                          params=search_param, _async=True, _callback=cb)
            status, _ = future.result()
            if status.OK():
                return
        assert False

    @pytest.mark.parametrize("collection", [[], None, "", 123])
    def test_search_in_segment_invalid_collection(self, collection, gcon):
        query_vectors = [[random.random() for _ in range(128)] for _ in range(100)]
        with pytest.raises(ParamError):
            gcon.search_in_segment(collection, file_ids=[1], top_k=1, query_records=query_vectors, params={"nprobe": 1})

    @pytest.mark.parametrize("ids", [[], None, "", 123])
    def test_search_in_segment_invalid_file_ids(self, ids, gcon):
        query_vectors = [[random.random() for _ in range(128)] for _ in range(100)]
        with pytest.raises(ParamError):
            gcon.search_in_segment("test", file_ids=ids, top_k=1, query_records=query_vectors, params={"nprobe": 1})

    @pytest.mark.parametrize("topk", [[], None, "", {}, True, False])
    def test_search_in_segment_invalid_topk(self, topk, gcon):
        query_vectors = [[random.random() for _ in range(128)] for _ in range(100)]
        with pytest.raises(ParamError):
            gcon.search_in_segment("test", file_ids=[1], top_k=topk, query_records=query_vectors, params={"nprobe": 1})

    @pytest.mark.parametrize("records", [[], None, "", 123, True, False])
    def test_search_in_segment_invalid_records(self, records, gcon):
        with pytest.raises(ParamError):
            gcon.search_in_segment("test", file_ids=[1], top_k=1, query_records=records, params={"nprobe": 1})

    @pytest.mark.parametrize("param", [[], "", 123, (), set(), True, False])
    def test_search_in_segment_invalid_param(self, param, gcon):
        query_vectors = [[random.random() for _ in range(128)] for _ in range(100)]
        with pytest.raises(ParamError):
            gcon.search_in_segment("test", file_ids=[1], top_k=1, query_records=query_vectors, params=param)



import concurrent.futures
import datetime
import random
import sys
import threading
import time

sys.path.append('.')
from factorys import time_it
from milvus import Milvus

dimension = 512
number = 100000
table_name = 'multi_task'


def add_vector_task(milvus, vector):
    status, ids = milvus.insert(table_name=table_name, records=vector)

    assert status.OK(), "add vectors failed"
    assert len(ids) == len(vector)


@time_it
def thread_pool_add_vector(milvus, pool_size, vectors):
    with concurrent.futures.ThreadPoolExecutor(max_workers=pool_size) as executor:
        for _ in range(pool_size):
            executor.submit(add_vector_task, milvus, vectors)


def test_run(gcon):
    gmilvus = gcon

    if gmilvus is None:
        assert False, "Error occurred: connect failure"

    status, exists = gmilvus.has_collection(table_name)
    if exists:
        gmilvus.drop_collection(table_name)
        time.sleep(2)

    table_param = {
        'collection_name': table_name,
        'dimension': dimension,
        'index_file_size': 1024,
        'metric_type': 1
    }

    gmilvus.create_collection(table_param)

    for p in (1, 5, 10, 20, 40, 50, 100):
        pool_size = p
    step = number // pool_size

    vectors = [[random.random() for _ in range(dimension)] for _ in range(step)]

    thread_pool_add_vector(gmilvus, p, vectors)

    time.sleep(1.5)
    _, gcount = gmilvus.count_entities(table_name)
    print(gcount)


def test_mult_insert(gcon):
    def multi_thread_opr(client, collection_name, utid):

        collection_param = {
            'collection_name': collection_name,
            'dimension': 64
        }

        vectors = [[random.random() for _ in range(64)] for _ in range(10000)]
        status = client.create_collection(collection_param)
        assert status.OK()

        status, _ = client.insert(table_name, vectors)
        assert status.OK()

    thread_list = []
    for i in range(10):
        t = threading.Thread(target=multi_thread_opr, args=(gcon, "multi_table_{}".format(random.randint(0, 10000)), i))
        t.start()
        thread_list.append(t)

    for tr in thread_list:
        tr.join(timeout=None)

    print("Done")



import pytest

from milvus import ParamError
from factorys import records_factory

dim = 128
nq = 100


class TestGetVectorByID:
    def test_get_entity_by_id(self, gcon, gcollection):
        vectors = records_factory(128, 1000)
        ids = [i for i in range(1000)]
        status, ids_out = gcon.insert(collection_name=gcollection, records=vectors, ids=ids)
        assert status.OK(), status.message

        gcon.flush([gcollection])

        status, vec = gcon.get_entity_by_id(gcollection, ids_out[0:10])
        assert status.OK()
        assert len(vec) == 10

        for v in vec:
            assert len(v) == 128

    @pytest.mark.parametrize("v_id", [None, "", [], {"a": 1}, (1, 2)])
    def test_get_entity_by_id_invalid_id(self, v_id, gcon):
        with pytest.raises(ParamError):
            gcon.get_entity_by_id("test_get_entity_by_id_invalid_id", [v_id])

    @pytest.mark.parametrize("collection", [None, -1, [], {"a": 1}, (1, 2)])
    def test_get_entity_by_id_invalid_collecton(self, collection, gcon):
        with pytest.raises(ParamError):
            gcon.get_entity_by_id(collection, [1])

    def test_get_entity_by_id_non_existent_collection(self, gcon):
        status, _ = gcon.get_entity_by_id("non_existent", [1])
        assert not status.OK()

    def test_get_entity_by_id_with_empty_collection(self, gcon, gcollection):
        status, _ = gcon.get_entity_by_id(gcollection, [1])
        assert not status.OK()

    @pytest.mark.parametrize("ids", [[0], [9999]])
    def test_get_entity_by_id_non_existent_id(self, ids, gcon, gvector):
        status, vector = gcon.get_entity_by_id(gvector, ids)
        assert status.OK()
        # assert not vector

    @pytest.mark.parametrize("v_tag", [1, [], {"a": 1}, (1, 2)])
    def test_get_entity_by_id_with_invalid_partition_tag(self, v_tag, gcon):
        with pytest.raises(ParamError):
            gcon.get_entity_by_id("test_get_entity_by_id_with_invalid_partition_tag", [1], None, v_tag)

    def test_get_entity_by_id_with_partition_tag(self, gcon, gcollection):
        vectors = records_factory(128, 1000)
        ids = [i for i in range(1000)]
        partition_tag = "test_get_entity_by_id_with_partition_tag"
        gcon.create_partition(gcollection, partition_tag)
        status, ids_out = gcon.insert(collection_name=gcollection, records=vectors, ids=ids, partition_tag=partition_tag)
        assert status.OK(), status.message

        gcon.flush([gcollection])

        status, vec = gcon.get_entity_by_id(gcollection, ids_out[0:10], None, partition_tag)
        assert status.OK()
        assert len(vec) == 10

        for v in vec:
            assert len(v) == 128

    def test_get_entity_by_id_with_non_existent_partition_tag(self, gcon, gcollection):
        vectors = records_factory(128, 1000)
        ids = [i for i in range(1000)]
        status, ids_out = gcon.insert(collection_name=gcollection, records=vectors, ids=ids)
        assert status.OK(), status.message

        gcon.flush([gcollection])

        status, vec = gcon.get_entity_by_id(gcollection, ids_out[0:10], None, "non-existent")
        assert not status.OK()
        assert len(vec) == 0


class TestDeleteByID:
    def test_delete_entity_by_id_normal(self, gcon, gcollection):
        vectors = records_factory(dim, nq)
        status, ids = gcon.insert(gcollection, vectors)
        gcon.flush([gcollection])
        assert status.OK()

        status = gcon.delete_entity_by_id(gcollection, ids[0:10])
        assert status.OK()

    @pytest.mark.parametrize("id_", [None, "123", []])
    def test_delete_entity_by_id_invalid_id(self, id_, gcon, gcollection):
        with pytest.raises(ParamError):
            gcon.delete_entity_by_id(gcollection, id_)

    @pytest.mark.skip
    def test_delete_entity_by_id_succeed_id(self, gcon, gcollection):
        vectors = records_factory(dim, nq)
        status, ids = gcon.insert(gcollection, vectors)
        assert status.OK()

        gcon.flush([gcollection])

        ids_exceed = [ids[-1] + 10]
        status = gcon.delete_entity_by_id(gcollection, ids_exceed)
        assert not status.OK()

    @pytest.mark.parametrize("v_tag", [1, [], {"a": 1}, (1, 2)])
    def test_delete_entity_by_id_with_invalid_partition_tag(self, v_tag, gcon):
        with pytest.raises(ParamError):
            gcon.delete_entity_by_id("test_delete_entity_by_id_with_invalid_partition_tag", [1], None, v_tag)

    def test_delete_entity_by_id_with_partition_tag(self, gcon, gcollection):
        vectors = records_factory(dim, nq)
        partition_tag = "test_delete_entity_by_id_with_partition_tag"
        gcon.create_partition(gcollection, partition_tag)
        status, ids = gcon.insert(gcollection, vectors, partition_tag=partition_tag)
        gcon.flush([gcollection])
        assert status.OK()

        status = gcon.delete_entity_by_id(gcollection, ids[0:10], None, partition_tag)
        assert status.OK()

    @pytest.mark.skip("It's ok to delete entity from non-existent partition now.")
    def test_delete_entity_by_id_with_non_existent_partition_tag(self, gcon, gcollection):
        vectors = records_factory(128, 1000)
        ids = [i for i in range(1000)]
        status, ids_out = gcon.insert(collection_name=gcollection, records=vectors, ids=ids)
        assert status.OK(), status.message

        gcon.flush([gcollection])

        status = gcon.delete_entity_by_id(gcollection, ids_out[0:10], None, "non-existent")
        assert not status.OK()


class TestGetVectorID:
    def test_get_vector_id(self, gcon, gvector):
        status, info = gcon.get_collection_stats(gvector)
        assert status.OK()

        seg0 = info["partitions"][0]["segments"][0]
        status, ids = gcon.list_id_in_segment(gvector, seg0["name"])
        assert status.OK()
        assert isinstance(ids, list)
        assert len(ids) == 10000

    @pytest.mark.parametrize("collection", [None, "", [], {"a": 1}, (1, 2), True, False])
    def test_get_vector_id_invalid_collection(self, collection, gcon):
        with pytest.raises(ParamError):
            gcon.list_id_in_segment(collection, "test")

    @pytest.mark.parametrize("segment", [None, "", [], {"a": 1}, (1, 2), True, False])
    def test_get_vector_id_invalid_segment(self, segment, gcon):
        with pytest.raises(ParamError):
            gcon.list_id_in_segment("test", segment)

    def test_get_vector_id_non_existent(self, gcon, gvector):
        status, _ = gcon.list_id_in_segment(gvector, "segment")
        assert not status.OK()

        status, info = gcon.get_collection_stats(gvector)
        assert status.OK()
        seg0 = info["partitions"][0]["segments"][0]
        status, _ = gcon.list_id_in_segment("test", seg0["name"])
        assert not status.OK()
